window.onload = function() {
  'use strict';

  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 8;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.names = [];
    this.results = {
      'practice trial': 0,
      'unanimous group': 0,
      'dissenter present': 0
    };
  };

  Trial.prototype.init = function() {
    this.createData();
    $(document).ready(function() {
      $('.t-1').hide();
      $('.t-2').hide();
      if (window.opener.platform.name === 'IE') {
        $('.exp2').css({
          'width': '100%',
          'height': '100%'
        });
        $('.trial-counter').css('top', '40%');
      }

      $('.pause-text-1').focus();
      $('body').prepend($('<div class="ada-shadow-speech" aria-live="polite"></div>'));
      if (window.opener.platform.os.family === 'iOS') {
        $('body').prepend($('<span id="focus_elem" tabindex="0" class="visually-hidden">Experiment Tab</span>'));
        setTimeout(function () {
          $('#focus_elem').focus();
        }, 2000);
      }
      trial.createButtons();
      if (window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android') {
        $('.press_instruction').html('Press the space bar or <span class="visually-hidden">double</span> tap<br/>to begin the experiment.');
        $('.experiment').bind('click', function () {
          trial.start('begin');
          $('.experiment').unbind('click');
        });

        for (var i = 1; i < 3; i++) {
          $('.pause-text-' + i).attr('aria-label', $('.pause-text-' + i).attr('aria-label').replace('Press the space bar', 'Press the space bar or double tap'));
        }
      }
    });

    document.onkeypress = this.startKeyListener.bind( this, 'begin' );
  };

  Trial.prototype.createButtons = function () {
    var buttonsStr = '';
    var startNum = 1;
    var endNum = 10;

    for (var i = startNum; i < endNum; i++) {
      buttonsStr += '<button id="button_' + i + '">' + i + '</button>';
    }
    $('.experiment').append($('<div class="button_container">' + buttonsStr + '</div>'));
    $('.button_container').find('button').each(function (i, button) {
      $(button).click(function () {
        trial.addAnswer(i + 1);
        if ( trial.i === 4) {
          trial.pause();
          trial.trialTextArray[trial.i].hide();
        }
        else {
          trial.showTrial();
        }
      });
    });

    $('.button_container').hide();
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    if ( keyCode === 32 ) {
      this.start( mode );
    }
  };

  Trial.prototype.createData = function() {
    for ( var i = 0; i < 8; i++ ) {
      this.data.push( { 'new': true } );
    }
    this.data.shuffle();
  };

  Trial.prototype.pause = function () {
    this.trialTextArray[8].hide();
    this.$trialCounter.hide();
    $('.t-1').hide();

    this.isMobile && this.$btnContainer.hide();

    this.$pauseText2.show();
    this.$textContainer.show();

    if (this.isIos) {
      this.setAriaFocusTo(this.$pauseText2);
    }
    else {
      this.$pauseText2.focus();
    }

    document.onkeypress = this.startKeyListener.bind( this, 'after pause' );
    if (this.isMobile) {
      $('.exp2').bind('click', function () {
        trial.start('after pause');
        $('.exp2').unbind('click');
      });
    }
  };

  Trial.prototype.start = function( mode ) {
    if ( mode === 'begin' ) {
      this.$btnContainer = $('.button_container');
      this.isAndroid = window.opener.platform.os.family === 'Android';
      this.isIos = window.opener.platform.os.family === 'iOS';
      this.isMobile = window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android';
      this.$liveRegion = $('.ada-shadow-speech');
      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounter = $( '.trial-counter' );
      this.trialTextArray = [];
      this.$pauseText1 = $( '.pause-text-1' );
      this.$pauseText2 = $( '.pause-text-2' );
      this.$textContainer = $( '.text-container' );
      this.$container = $( '.application-container' );
      this.$closeButton = $( '.close-button' );
      this.$membersAnswers = $( '.members-answers' );
      for ( var i = 1; i <= 8; i++ ) {
        $( '.trial-text-'+i ).find('br').remove();
        this.trialTextArray[i] = $( '.trial-text-'+i );
      }
      $('.t-1').show();
    }
    else {
      this.$membersAnswers.show();
      $('.t-2').show();
    }
    this.$pauseText1.hide();
    this.trialTextArray[this.i+1].show();
    this.$trialCounter.show();
    this.$textContainer.hide();
    this.$pauseText2.hide();

    if (this.isMobile) {
      $('.button_container').show();
    }

    document.onkeypress = this.keyListener.bind( this );
    this.showTrial();
  };

  Trial.prototype.saveResult = function() {
    this.$trialCounter.hide();
    this.trialTextArray[8].hide();
    $('.trial-text').hide();
    this.$membersAnswers.hide();
    this.$closeButton.show();
    this.$closeButton.find('button').focus();
    if (this.isMobile) {
      this.$btnContainer.hide();
    }
    $('.t-1').hide();
    $('.t-2').hide();

    var i;
    this.results = {
      'ended_benefits': this.answers[0],
      'ended_costs': this.answers[1],
      'ended_alternatives': this.answers[2],
      'ended_investment': this.answers[3],
      'successful_benefits': this.answers[4],
      'successful_costs': this.answers[5],
      'successful_alternatives': this.answers[6],
      'successful_investment': this.answers[7]
    };

     window.results = this.results;
  };

  Trial.prototype.showTrial = function() {
    if ( this.i < this.trialNum ) {
      var html = '';
      this.$membersAnswers.html( html );
      this.i++;
      this.$trialCounterCurrent.text( this.i );
      var instText = this.i === 1 ? $('.t-1').text() : this.i === 5 ? $('.t-2').text() : '';

      if (this.isIos) {
        this.$liveRegion.html(this.$trialCounter.text() + ' ' + instText + ' ' + $('.trial-text-' + this.i).text());
      } else {
        this.$liveRegion.html(this.$trialCounter.text() + ' ' + instText);
      }

      this.trialTextArray[this.i].show();
      $('.trial-text-' + this.i).focus();
      if ( this.trialTextArray[this.i - 1] !== undefined ) {
        this.trialTextArray[this.i - 1].hide();
      }
    }
    else {
      this.saveResult();
    }
  };

  Trial.prototype.addAnswer = function( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function( event ) {
    var keyCode = event.keyCode || event.which;
    var str;

    str = String.fromCharCode( keyCode );

    if ( '998877665544332211'.indexOf( str ) > -1 ) {
      ( str === '9' || str === '9' ) && ( this.addAnswer( 9 ) );
      ( str === '8' || str === '8' ) && ( this.addAnswer( 8 ) );
      ( str === '7' || str === '7' ) && ( this.addAnswer( 7 ) );
      ( str === '6' || str === '6' ) && ( this.addAnswer( 6 ) );
      ( str === '5' || str === '5' ) && ( this.addAnswer( 5 ) );
      ( str === '4' || str === '4' ) && ( this.addAnswer( 4 ) );
      ( str === '3' || str === '3' ) && ( this.addAnswer( 3 ) );
      ( str === '2' || str === '2' ) && ( this.addAnswer( 2 ) );
      ( str === '1' || str === '1' ) && ( this.addAnswer( 1 ) );
      if ( this.i === 4) {
        this.pause();
        this.trialTextArray[this.i].hide();
      }
      else {
        this.showTrial();
      }
    }
  };

  Trial.prototype.setAriaFocusTo = function ($elem, delay) {
    delay = delay || 0;

    setTimeout(function () {
      $elem = typeof $elem === 'string' ? $($elem) : $elem;

      if (this.isIos && $elem.find('*').length) {
        $elem.find('.ios-vo-focus-fix').remove();
        // $elem.find('.visible-part').remove();

        $elem.css('position', 'relative');
        var $elemText = $elem.attr('aria-label') || $elem.text();
        var itemDescribedByText = $('#' + $elem.attr('aria-describedby')).text() || '';
        var $iosVoFocusableElem = $('<div class="ios-vo-focus-fix" tabindex="0">' + $elemText + ' ' + itemDescribedByText + '</div>');
        if (!$elem.find('.visible-part').length) {
          $elem.html('<div aria-hidden="true" class="visible-part">' + $elem.html() + '</div>');
        }
        $elem.prepend($iosVoFocusableElem);
        $elem = $iosVoFocusableElem;
      }

      $elem.focus();
    }.bind(this), delay);
  };

  var trial = new Trial();

  trial.init();
};
