/**
 * @fileoverview Object for all common code between modules
 * If code is not used at least twice it should not be here
 * You should update this file in mac_simpleframework if you
 * add something to this file
 */

/**
 * Set us up the CommonAll object
 * 
 * @constructor
 */
function CommonAll(){
	var _keyDownVal = "";
	var _keyUpVal = "";
	//Set public functions here
	return{
		/**
		 * A generic debug object
		 * @param {object} arguments (optional) - object of arguments
		 */
		debug : function(arguments){
			$('.appContainer').addClass('debug');
		},
		/**
		 * Use this to log stuff
		 */
		log : function(logString){
			if(window.console){
				console.log(logString);
			}
		},
		/**
		 * Transforming results from format: [{"name":"Name","value":"Value"},...] to [{"Name":"Value"}]
		 * @param results
		 */
		normalizeResults : function(results){
		    var _return={};
		    for (var i=0; i<results.length; i++) {
		        _return[results[i]["name"]] = results[i]["value"];
		    }
		    return _return;
		},
		checkNumeric: function(element,min,max,even,evenErrorMsg,boundsErrorMsg){
			element.change({min:min,max:max,even:even,evenErrorMsg:evenErrorMsg,boundsErrorMsg:boundsErrorMsg},function(e){	
				var showError = false;
				
				//see if bigger or smaller than min-max
				if(isNaN(e.currentTarget.value) || parseInt(e.currentTarget.value) < e.data.min){
					e.currentTarget.value = e.data.min;
					
					if(e.data.boundsErrorMsg){
						var $dialog = errorDialog(e.data.boundsErrorMsg);					
						$dialog.dialog('open');
					}
					
					showError = true;
				}
				else if(parseInt(e.currentTarget.value) > e.data.max){
					e.currentTarget.value = e.data.max;
					
					if(e.data.boundsErrorMsg){
						var $dialog = errorDialog(e.data.boundsErrorMsg);					
						$dialog.dialog('open');
					}
					
					showError = true;
				}
				//check to see if even if even and errorMsg are true
				if(even && evenErrorMsg && e.currentTarget.value % 2 != 0){
					//not even bump it by 1 to make it even and return
					e.currentTarget.value++;
					if(!showError){
						var $dialog = errorDialog(e.data.evenErrorMsg);					
						$dialog.dialog('open');
					}
				}
				function errorDialog(text){
					return $('<div></div>').html(text)
					                       .dialog({autoOpen: false,
                                                    title: 'Error',
                                                    resizable: false,
                                                    modal: true,
                                                    draggable: false,
                                                    buttons: { "OK": function(){$(this).dialog("close");}}});
				}
			});
		},

        /**
         * Binds slider functionality to an element
         *
         * @param element - slider element
         * @param min - minimum slider value
         * @param max - maximum slider value
         * @param value - default slider value
         * @param square - element to apply slider value to
         * @param input - hidden input element to hold the value
         */
        makeSlider: function(element, min, max, value, square, input){
            element.slider({
                min: min,
                max: max,
                value: value,
                slide: function(event, ui) {
                    square.css("background", "rgb("+ui.value+","+ui.value+","+ui.value+")");
                    input.val(ui.value);
                }
            });
        },
        
		//returns a random # between 0 and length-1
		randomIndex: function(length){
			return Math.floor(Math.random()*length)
		},
		
        //Merge 2 objects
        mergeRecursive: function(obj1, obj2){
            for(var p in obj2) {
                if ( obj2[p].constructor==Object ) {
                    obj1[p] = this.mergeRecursive(obj1[p], obj2[p]);
                } else {
                    obj1[p] = obj2[p];
                }
            }
            return obj1;
        }
	};
}