/**
 * @fileoverview Object for all common code between advanced settings
 * If code is not used at least twice it should not be here
 * You should update this file in mac_simpleframework if you
 * add something to this file
 */

/**
 * Set us up the CommonAdvancedSettings object
 *
 * @constructor
 */
function CommonExperiment() {

    /**
     * Holds raw data for start and finish time of all events
     * @type {Object}
     * @private
     */
    var _events = {},
        //Holds timing data according to a storyboard.
        _storyBoard = {},
        //System time marking beginning of a trial
        _startTime = 0,
        //holds the time for each event
        _timeline = {},      
        //flag to determine if need to fallback to flash in IE8
        _isIE8,
    /*
     * Object holding initialized sounds variable
     */
        _soundObject = {},
    /*
     * Lock
     */
        _isShowDelay = false,
        _farSoundPlayed = false,
    /*
     * HTML elements
     */
        _square = null,
        _dot = null,
    //sound indicator
        _isPlayingSound = false,
        _sound1000Timer = false,
        _animationTimer = false,
        _animationFps      = 30,
    //start angle
        _currentAngle      = 30,
        _container      = null,

    //Generic sound variables
        _genSound;
    
    /**
     * Displays text in the center of the App Container that can
     * be continued via a keypress
     *
     * @param {object} appContainer (required) - the application container
     * @param {array} text (required) - array of strings to display.
     *     Array.length*lineHeight determines box height
     * @param {int} textWidth - width (in pixels) of the text container
     * @param {string} charCode (optional) - key to advance screen
     * @param {function} callback (required with charCode) - function to execute after keypress
     *     This will not be executed if charCode is not specified
     */
    var dspTextKey = function (appContainer, text, boxWidth, charCode, callback, boxID, boxClass) {
            var boxID = Boolean(boxID) ? boxID : 'experiment';
            var boxClass = Boolean(boxClass) ? boxClass : 'textBox';

            //make the box to hold the text
            makeTextBox(appContainer, boxWidth, text, boxID, boxClass);

            //set up the keypress listener if a key is specified
            if (charCode != undefined && charCode != -1) {
                //verify callback, fail otherwise
                if (callback) {
                    $(document).keypress({charNeeded:charCode, callback:callback}, function (e) {
                        if (e.data.charNeeded == e.which) {
                            $(document).unbind(e);
                            callback();
                        }
                    });
                }
                else {
                    //throw a simple exception
                    $.error('Callback must be defined with charCode');
                }
            }
            //else if -1 pass data directly to callback
            else if (charCode == -1) {
                $(document).keypress({callback:callback}, function (e) {
                    callback(e);
                });
            }
        },
        /**
         * Displays text in the center of the App Container that can
         * be continued via true/false keypress
         */
        makeTrueFalseBox = function (appContainer, boxWidth, text, trueBtnText, falseBtnText, boxID, boxClass, padding, reverse, displayTimeout,logging,appDate,appSystemTime) {
            var lineHeight = parseInt(appContainer.css("line-height")), //line height will be used to calculate height
                dspText = '',
                padding = Boolean(padding) ? padding : 0;

            //padding for the top is padding+1 to account for buttons
            for (var i = 0; i < padding + 1; i++) {
                dspText += '<br/>';
            }

            //Add a span around the text so we can show/hide it
            dspText += '<span id="txtHolder">';

            //loop the text, build the string
            var i = 0;
            for (var lines in text) {
                if (Boolean(i)) {
                    dspText += ('<br/>' + text[lines]);
                } else {
                    dspText += text[lines]
                }
                i++;
            }

            dspText += '</span>';

            //padding for the top is padding+1 to account for buttons
            for (var j = 0; j < padding; j++) {
                dspText += '<br/>';
            }

            var boxHeight = (((padding * 2) + i) * lineHeight) + 40;
            var boxX = (appContainer.width() - boxWidth) / 2; //find boxX pos
            var boxY = (appContainer.height() - boxHeight) / 2; //find boxY pos

            //define the text holder
            var textBox = $('<div/>').attr('id', boxID)
                .addClass(boxClass)
                .css({width:boxWidth,
                    height:boxHeight,
                    position:"absolute",
                    top:boxY,
                    left:boxX})
                .html(dspText);

            var trueItem = $('<button/>').attr('id', 'true')
                .addClass('choiceBtn')
                .val(1)
                .html(trueBtnText);

            //check for reverse
            if (!reverse) {
                textBox.append(trueItem);
            }
            textBox.append($('<button/>').attr('id', 'false')
                .addClass('choiceBtn')
                .val(0)
                .html(falseBtnText));
            if (reverse) {
                textBox.append(trueItem);
            }

            //log the word display if logging
            if(logging){
            	this.eventStart(text[0],appDate.getTime()-appSystemTime);
            	if(window.console){
            		console.log(text[0]+" displayed");
    			}   	
            }
            
            //append the buttons
            appContainer.append(textBox);

            //if timeout is set set to visible:hidden after time met
            if (Boolean(displayTimeout)) {
                setTimeout(function () {
                    $('#txtHolder').css('visibility', 'hidden');
                    //log the word display if logging
                    if(logging){
                    	eventFinish(text[0],appDate.getTime()-appSystemTime);
                    	if(window.console){
                    		console.log(text[0]+" removed in "+displayTimeout+"ms");
                    	}
                    }
                }, displayTimeout);
            }
        },
        /**
         * Make a centered text box in the middle of your application screen
         *
         * @param {object} appContainer (required) - container of the application
         * @param {int} boxWidth (required) - box width
         * @param {array} text (required) - array of strings to display.
         *     Box is calculated text.lengh*body.line-height
         * @param {string} boxClass (required) - box class for the box
         */
        makeTextBox = function (appContainer, boxWidth, text, boxID, boxClass) {
            var lineHeight = parseInt(appContainer.css("line-height")); //line height will be used to calculate height
            var dspText = "";

            //loop the text, build the string
            var i = 0;
            for (var lines in text) {
                if (Boolean(i)) {
                    dspText += ('<br/>' + text[lines]);
                } else {
                    dspText += text[lines]
                }
                i++;
            }

            var boxHeight = i * lineHeight;
            var boxX = (appContainer.width() - boxWidth) / 2; //find boxX pos
            var boxY = (appContainer.height() - boxHeight) / 2; //find boxY pos

            //define the text holder
            var textBox = $('<div/>').attr('id', boxID)
                .attr('tabindex', 0)
                .attr('aria-label', dspText.replace(/<br\/?>/g, ' '))
                .addClass(boxClass)
                .css({width:boxWidth,
                    height:boxHeight,
                    position:"absolute",
                    top:boxY,
                    left:boxX})
                .html(dspText);

            appContainer.append(textBox);
        },
        /**
         * Show a focus point on the screen for (time) amount
         * of time
         *
         * @param {int} currTime (required) - curr application time ms
         * @param {object} container (required) - application container
         * @param {string} boxID (required) - ID of the box that holds text on screen
         * @param (int) removeFocus (optional) - focal point removal time in ms
         * @param (int) stop (optional) - stop time in ms
         *
         * @return false if time to stop, true if time to continue
         */
            showFocusPoint = function (currTime, container, boxID, removeFocus, stop) {
            var keepGoing = true,
                removeFocus = typeof removeFocus == "number" ? removeFocus : 750,
                stop = typeof stop == "number" ? stop : 1000;

            if (!currTime) {
                container.children('#' + boxID).html('+');
            }
            else if (currTime == removeFocus) {
                container.children('#' + boxID).html('');
            }
            else if (currTime >= (removeFocus + stop)) {
                keepGoing = false;
            }

            return keepGoing;
        },
        /**
         * Swap text for other text in an array of strings
         *
         * @param {string} value (required) - value to swap for
         * @param {string} needle (required) - value to search for
         * @param {array} haystack (required) - value to search in
         *
         * @return array of strings with item swapped
         */
        swapText = function (value, needle, haystack) {
            for (var i in haystack) {
                haystack[i] = haystack[i].replace(needle, value);
            }
            return haystack;
        }, 
        /**
         * Initialize the experiment timing variable
         */
        startExperimentTiming = function () {
            _startTime = new Date().getTime();
        },

        /**
         *
         * Mark the beginning of an event
         *
         * @param {string} eventName (required) - Name of the event
         * @param {int} currTime (required) - Current loop time
         */
        eventStart = function(eventName, currTime){
            var tmpTime = new Date().getTime();
            
            if (typeof(_events[eventName]) == "undefined") {
                _events[eventName] = {start:[], finish:[]};
            }
            if(typeof(_timeline[eventName]) == "undefined") {
            	_timeline[eventName] = {start:0, finish:0};
            }
            if (typeof(_storyBoard[eventName]) == "undefined") {
                _storyBoard[eventName] = {start:0, finish:0};
            }
            _storyBoard[eventName].start = currTime;
            _events[eventName].start.push((tmpTime - _startTime));
            _timeline[eventName].start = tmpTime-_startTime;
        },

        /**
         *
         * Mark the finish of an event
         *
         * @param {string} eventName (required) - Name of the event
         * @param {int} currTime (required) - Current loop time
         */
        eventFinish = function (eventName, currTime) {
        	var tmpTime = new Date().getTime();
        	
            if (typeof(_events[eventName]) == "undefined") {
                return false;
            }
            _storyBoard[eventName].finish = currTime;
            _events[eventName].finish.push(parseInt(new Date().getTime() - _startTime));
            _timeline[eventName].finish = tmpTime-_startTime;
        },
        /**
         * Can be used in combined experiments to track the time separately
         */
        resetTimingResults = function () {
            _events = {};
            _storyBoard = {};
            _startTime= 0;
        },
        /**
         * Logs testing data to the browser's console in CSV format
         */
        dumpTestingDataCSV = function () {
            var output = '';

            var longestArrayLength = 0;

            //Experimental data
            output += '"Experimental_data"';

            for (event in _events) {
                output += ';"' + event + '"';
                if (_events[event].start.length > longestArrayLength) {
                    longestArrayLength = _events[event].start.length;
                }
            }

            output += '\n';

            for (var k = 0; k < longestArrayLength; k++) {
                var row = [null];

                for (event in _events) {
                    if (_events[event].start[k] == null) {
                        row.push(null);
                    } else {
                        row.push((_events[event].finish[k] - _events[event].start[k]));
                    }
                }
                output += row.join(";");
                output += '\n';
            }

            output += '"Story board timing"';
            for (event in _events) {
                output += ';"' + event + '"';
            }

            output += '\n';

            var row = [null];
            for (event in _events) {
                row.push(_storyBoard[event].finish - _storyBoard[event].start);
            }

            output += row.join(";");


            console.log("==========EXPERIMENT DATA IN CSV FORMAT==============");
            console.log(output);
            console.log("==========EXPERIMENT DATA IN CSV FORMAT==============");
        },
        /**
         * Dumps event by event data based on time stamps set with eventStart/Finish
         * to use this dump you will need to call methods in the following order:
         * 
         * When your experiment is ready call:
         * startExperimentTiming();
         * 
         * Then call:
         * eventStart();
         * at the beginning of you event that you need timed
         * 
         * Then call:
         * eventFinish();
         * after your event has completed to record the finish time
         * 
         * When your experiment finishes call this method to dump
         * AmChart data to the console
         * 
         * @param - [limit] - int - returns [limit] numbers of results
         * data to the console. Leave blank for all.
         */
        dumpTestingTimelineData = function(limit){
        	var formattedResults = [],
        		i = 0;
        	
        	//loop the results and build the chart data
        	for(var line in _timeline){
        		//if rounds are supplied and met break out
        		if(Boolean(limit) && i >= limit){
        			break;
        		}
        		
        		//build the results array
        		formattedResults.push({name: line,
        			                   startTime: _timeline[line].start,
        			                   endTime: _timeline[line].finish,
        			                   value: _timeline[line].finish-_timeline[line].start,
        			                   color: "#"+(Math.floor(Math.random() * 16777215).toString(16))});
        		
        		//up the counter
        		i++;
        	}
        	
        	//encode to json and dump to console
        	if(typeof console != "undefined"){
        		console.log("==========AMCHARTS JS OBJECT START==============");
        		console.log("var chartData = JSON.parse('"+JSON.stringify(formattedResults)+")';");
        		console.log("==========AMCHARTS JS OBJECT FINISH==============");
        	}
        },
        /**
         * Log testing data in a format suitable for AmCharts
         */
        dumpTestingDataCharts = function () {
            var outputAverage = 'var chartData = [';
            var outputRandom = 'var chartData = [';
            var first = true;
            for (event in _events) {
                var averageStart = 0,
                    averageFinish = 0,
                    randomStart = 0,
                    randomFinish = 0;

                for (var i = 0; i < _events[event].start.length; i++) {
                    averageStart += _events[event].start[i];
                    averageFinish += _events[event].finish[i];
                }

                averageStart = parseInt(averageStart / _events[event].start.length);
                averageFinish = parseInt(averageFinish / _events[event].start.length);

                randomStart = _events[event].start[Math.floor(Math.random() * (_events[event].start.length))];
                randomFinish = _events[event].finish[Math.floor(Math.random() * (_events[event].finish.length))];

                var sign = ",";
                if (first) {
                    sign = ""
                    first = false;
                }
                outputAverage += (sign + "{ name: '" + event + "',");
                outputRandom += (sign + "{ name: '" + event + "',");

                outputAverage += "startTime: " + averageStart + ",";
                outputAverage += "endTime: " + averageFinish + ",";
                outputAverage += "value: " + (averageFinish - averageStart) + ",";

                outputRandom += "startTime: " + randomStart + ",";
                outputRandom += "endTime: " + randomFinish + ",";
                outputRandom += "value: " + (randomFinish - randomStart) + ",";

                outputAverage += "color: '#" + (Math.floor(Math.random() * 16777215).toString(16)) + "' }";
                outputRandom += "color: '#" + (Math.floor(Math.random() * 16777215).toString(16)) + "' }";
            }
            outputAverage += "];";
            outputRandom += "];";
            console.log("==========AMCHARTS JS OBJECT START (AVERAGE RESULTS)==============");
            console.log(outputAverage);
            console.log("==========AMCHARTS JS OBJECT FINISH (AVERAGE RESULTS)==============");
            console.log("==========AMCHARTS JS OBJECT START (RANDOM RESULTS)==============");
            console.log(outputRandom);
            console.log("==========AMCHARTS JS OBJECT FINISH (RANDOM RESULTS)==============");
        },
        /**
         * Create html layout used in excercise
         */
        initBallExercise = function (_appContainer) {
            _container = _appContainer;
            _container.append('<div id="delayShowBallAndBox" style="display:none;"><div id="dot"></div><div id="square"></div></div>');

            _dot = $("#dot");
            _square = $("#square");
            _dot.css ({
                top: _container.height()/2,
                left: _container.width()/2
            });
            _square.draggable({
                start: function() {
                    assesSquareDistance();
                },
                drag: function() {
                    assesSquareDistance();
                },
                stop: function() {
                    assesSquareDistance();
                }
            });
        },
        /**
         * Sound initialization (with fallback in IE8)
         */
        initSound = function () {
            if ($.browser.msie==true &&
                $.browser.version.substring(0, 2) == "8.") {
                _isIE8 = true;
            } else {
                _isIE8 = false;
            }

            if ($.browser.msie==true) {
                var _sound1000File = "1000hz.mp3";
                var _sound2000File = "2000hz.mp3";
            } else {
                var _sound1000File = "1000hz.wav";
                var _sound2000File = "2000hz.wav";
            }


            if (!_isIE8) {
                _soundObject      = {
                    1000: new Audio("./sounds/"+_sound1000File),
                    2000: new Audio("./sounds/"+_sound2000File)
                }
            } else {
                $.getScript("js/soundmanager2.js", function(data, textStatus, jqxhr) {
                    //console.log(data); //data returned
                    //console.log(textStatus); //success
                    //console.log(jqxhr.status); //200
                    //console.log('Load was performed.');

                    _soundObject = {};
                    soundManager.setup({
                        url: 'swf',
                        flashVersion: 9,
                        onready: function() {
                            var options1000 = soundManager.createSound({
                                id: 'tone1000',
                                url: 'sounds/'+_sound1000File
                            });
                            options1000.load();
                            var options2000 = soundManager.createSound({
                                id: 'tone2000',
                                url: 'sounds/'+_sound2000File
                            });
                            options2000.load();
                            _soundObject = {
                                1000: options1000,
                                2000: options2000
                            };
                        },
                        defaultOptions: {
                            // set global default volume for all sound objects
                            volume: 33
                        }
                    });
                });
            }
        },
        /**
         * Start playing
         *
         * @param {string} hz (required) - 1000/2000 Hz sound
         */
        playSound = function (hz) {
            if (_isIE8) {
                _soundObject[hz].togglePause()
            } else {
                _soundObject[hz].play();
            }
        },
        /**
         * Pause currently playing sound
         *
         * @param {string} hz (required) - 1000/2000 Hz sound
         */
        pauseSound = function(hz) {
            if (_isIE8) {
                _soundObject[hz].togglePause()
            } else {
                _soundObject[hz].pause();
            }
        },
        /**
         * Sound initialization for generic sound file
         * 
         * @param - sound - the file to play for all modern browsers
         * @param - legacySound -  the file to play for all legacy browsers
         */
        initGenSound = function(sound,legacy) {   
        	//check for oldIE
            if ($.browser.msie==true &&
                $.browser.version.substring(0, 2) == "8.") {
                _isIE8 = true;
            }
            else if($.browser.msie==true &&
                    $.browser.version.substring(0, 2) == "9."){
            	_isIE9 = true;
            	_isIE8 = false;
            }
            else {
                _isIE8 = false;
            }

            if(!_isIE8) {
            	if(_isIE9){
            		_genSound = new Audio(legacy);
            	}
            	else{
            		_genSound = new Audio(sound);
            	}
            }
            else{
            	$.getScript("js/soundmanager2.js", function(data, textStatus, jqxhr) {
                    soundManager.setup({
                        url: 'swf',
                        flashVersion: 9,
                        onready: function() {
                            _genSound = soundManager.createSound({
                                id: 'genSound',
                                url: legacy
                            });
                            _genSound.load();
                        },
                        defaultOptions: {
                            volume: 33
                        }
                    });
                });
            }
        },
        /**
         * Start playing
         *
         * @param {string} hz (required) - 1000/2000 Hz sound
         */
        playGenSound = function() {
            if (_isIE8) {
            	_genSound.togglePause()
            } else {
            	_genSound.play();
            }
        },
        /**
         * Pause currently playing sound
         *
         * @param {string} hz (required) - 1000/2000 Hz sound
         */
        pauseGenSound = function() {
            if (_isIE8) {
            	_genSound.togglePause()
            } else {
            	_genSound.pause();
            }
        },
        /**
         * find the square distance from the ball
         */
        assesSquareDistance = function () {
            if (!_isShowDelay) {
                return false;
            }
            //Check if the dot is inside the square
            if (
                ( parseInt(_dot.css("top"))>=parseInt(_square.css("top")) &&
                    (parseInt(_dot.css("top"))+4)<=(parseInt(_square.css("top"))+25) )   &&
                    (
                        parseInt(_dot.css("left"))>=parseInt(_square.css("left")) &&
                            (parseInt(_dot.css("left"))+4)<=(parseInt(_square.css("left"))+25)
                        )
                ) {
                _square.css("border-color", "cyan");
                _farSoundPlayed=false;
            } else {
                var x1 = parseInt(_dot.css("left"))+2;
                var y1 = parseInt(_dot.css("top"))+2;
                var x2 = parseInt(_square.css("left"));
                var y2 = parseInt(_square.css("top"));

                if (x1 > x2) {
                    x2 += 25;
                }
                if (y1 > y2) {
                    y2 += 25;
                }

                var distance = Math.sqrt( ( Math.pow((y2-y1),2) + Math.pow((x2-x1),2) ) );
                if (distance < 4*10) {
                    _farSoundPlayed=false;
                    _square.css("border-color", "yellow");
                } else {
                    _square.css("border-color", "red");
                    if (!_isPlayingSound) {
                        playSound(2000);
                        _isPlayingSound=true;
                        setTimeout(function() {
                            _isPlayingSound = false;
                        }, 550);
                    }
                }
            }
        },
        /**
         * start exercise
         */
            showBallExercise = function () {
            $("#delayShowBallAndBox").css("display", "block");
            _isShowDelay = true;
            var currentSoundTime=0
            _sound1000Timer = setInterval(function(){
                playSound(1000);
            }, 1000);
            var animationCounter=0;
            _animationTimer = setInterval(function(){
                var coords = getNewCoords(parseInt(_dot.offset().left), parseInt(_dot.offset().top));
                _dot.css({
                    "left": parseInt(_dot.css("left"))+coords.left,
                    "top": parseInt(_dot.css("top"))+coords.top
                });
                animationCounter += parseInt(1000/_animationFps);
                if (animationCounter % (parseInt(1000/_animationFps)*3) == 0) {
                    assesSquareDistance();
                }
            }, 1000/_animationFps);
        },
        /**
         * calculate coordinates of the dot
         */
            getNewCoords = function (x, y) {
            var _dotSpeed          = 4;
            if ( x>=($(_container).offset().left+$(_container).width()) || x<=$(_container).offset().left ) {
                if (_currentAngle==90) {
                    _currentAngle=270;
                } else if (_currentAngle<=180) {
                    _currentAngle = 180-_currentAngle;
                } else if (_currentAngle==270) {
                    _currentAngle = 90;
                } else if (_currentAngle>180) {
                    _currentAngle = 360+(180-_currentAngle);
                }
            } else if (y <= $(_container).offset().top || y>=($(_container).height()+$(_container).offset().top)) {
                if (_currentAngle==90) {
                    _currentAngle=270;
                } else if (_currentAngle<=180) {
                    _currentAngle = 360-_currentAngle;
                } else if (_currentAngle==270) {
                    _currentAngle = 90;
                } else if (_currentAngle>180) {
                    _currentAngle = 360-_currentAngle;
                }
            } else {
                if ( (Math.floor(Math.random()*2) + 1) == 2) {
                    _currentAngle += 30;
                } else {
                    _currentAngle -= 30;
                }

                if (_currentAngle<0) {
                    _currentAngle = 330;
                } else if (_currentAngle==360) {
                    _currentAngle = 0;
                }
            }

            if (_currentAngle==0) {
                return {left: _dotSpeed, top: 0};
            } else if (_currentAngle==30) {
                return {left: _dotSpeed*0.866, top: -(_dotSpeed/2)}
            } else if (_currentAngle==60) {
                return {left: _dotSpeed/2, top: -(_dotSpeed*0.866)}
            } else if (_currentAngle==90) {
                return {left: 0, top: -(_dotSpeed)}
            } else if (_currentAngle==120) {
                return {left: -(_dotSpeed/2), top: -(_dotSpeed*0.866)}
            } else if (_currentAngle==150) {
                return {left: -(_dotSpeed*0.866), top: -(_dotSpeed/2)}
            } else if (_currentAngle==180) {
                return {left: -(_dotSpeed), top: 0}
            } else if (_currentAngle==210) {
                return {left: -(_dotSpeed*0.866), top: (_dotSpeed/2)}
            } else if (_currentAngle==240) {
                return {left: -(_dotSpeed/2), top: (_dotSpeed*0.866)}
            } else if (_currentAngle==270) {
                return {left: 0, top: (_dotSpeed)}
            } else if (_currentAngle==300) {
                return {left: (_dotSpeed/2), top: (_dotSpeed*0.866)}
            } else if (_currentAngle==330) {
                return {left: (_dotSpeed*0.866), top: (_dotSpeed/2)}
            } else {
                return {left: _dotSpeed, top: 0};
            }
        },
        /**
         * Stop exercise
         */
        hideBallExercise = function () {
            $("#delayShowBallAndBox").css("display", "none");
            _isShowDelay = false;
            clearInterval(_sound1000Timer);
            clearInterval(_animationTimer);
            pauseSound(1000);
            pauseSound(2000);
            _sound1000Timer=false;
            _animationTimer=false;
        };



    //public methods go here
    return{
        dspTextKey:dspTextKey,
        makeTrueFalseBox: makeTrueFalseBox,
        makeTextBox:makeTextBox,
        showFocusPoint:showFocusPoint,
        swapText:swapText,
        eventStart:eventStart,
        eventFinish:eventFinish,
        dumpTestingDataCSV:dumpTestingDataCSV,
        dumpTestingDataCharts:dumpTestingDataCharts,
        dumpTestingTimelineData:dumpTestingTimelineData,
        startExperimentTiming:startExperimentTiming,
        initSound: initSound,
        playSound: playSound,
        pauseSound: pauseSound,
        initBallExercise: initBallExercise,
        showBallExercise: showBallExercise,
        hideBallExercise: hideBallExercise,
        resetTimingResults: resetTimingResults,
        initGenSound: initGenSound,
        playGenSound: playGenSound,
        pauseGenSound: pauseGenSound
    };
}