/**
 * @fileoverview Handles loading of experiment datasets from advanced_settings.xml
 * Called from factory.js
 *
 * @todo Get this running of a base object for common methods like setData
 */

/**
 *
 * @param {object} data (required)      - XML data to set
 * @param {string} container (optional) - ID of the container for html output
 * @constructor
 */
var AdvancedSettingsParser = function(data,container){
  this._data       = undefined;
  this._dataObject = {};
  this._xml        = undefined;
  this._container  = 'advancedSettings';
  this._html       = undefined;

  //Set the data
  this.setData(data);

  //Set the container
  this.setContainer(container);
};

/**
 * Load the object with a new XML object
 *
 * @param {object} data (required) - New XML data to set
 */
AdvancedSettingsParser.prototype.setData = function(data){
  //Set data and xml object
  if(data){
    this._data = data;
    this._xml  = $(this._data);
  }
  else{
    //@todo exception on failure
    alert('ExperimentParser.setData needs data');
  }
};

/**
 * Set the html container for drawing HTML from XML
 * @todo Make this part of a base object
 *
 * @param {string} container (required) - ID of the container
 */
AdvancedSettingsParser.prototype.setContainer = function(container){
  /**
   * set up the container for HTML output. If it doesn't exist we use the
   * default for the document
   */
  if(container && typeof container === "string"){
    this._container = $('#'+container);
  }
  else{
    //@todo Could throw an exception here
    this._container = $('#'+this._container);
  }
};

/**
 * Builds a Form Object and returns it for display
 * Also sets the Form to _html on this Object
 */
AdvancedSettingsParser.prototype.buildHTML = function(CommonAll){
  var _CommonAll = CommonAll;
  var tmpDataObject = this._xml.children('form');

  //loop the data elements
  for(var i = 0; i <  tmpDataObject.length; i++){
    var currData = $(tmpDataObject[i]),
      tmpDataObjectChildren = currData.children();

    //set us up the form
    var tmpForm = $('<form/>').attr({'method': 'post',
      'class': currData.attr('class'),
      'id': currData.attr('id')});

    //no recursion needed here since the DOM is predictable
    for(var j = 0; j <  tmpDataObjectChildren.length; j++){
      var tempData = $(tmpDataObjectChildren[j]);

      for(var k = 0; k <  tempData.length; k++){
        //if it's body we're looping elements
        if(tempData[k].tagName == "body"){
          elementParser(tmpForm,$(tempData[k]).children());
        }
        //otherwise dealing with a simple object
        else if(tempData[k].tagName == "header"){
          headerParser(tmpForm,$(tempData[k]).children());
        }
      }
    }
  }

  //send it back for display
  this._html = tmpForm;
  return tmpForm;

  /**
   * Parser for header data
   *
   * @private
   */
  function headerParser(element,data){
    var formRow = $('<div/>').addClass('headerElem');
    var tmpElem;
    var currItem;

    for(var i = 0; i < data.length; i++){
      currItem = $(data[i]);
      tmpElem  = undefined;

      //figure out which element we're dealing with
      //<header><title>
      if(currItem.prop('tagName') == "title" && currItem.text() != ""){
        //Check if header size is defined and in range
        if(currItem.attr('size') && currItem.attr('size') >= 1 && currItem.attr('size') <= 6){
          tmpElem = $('<h'+currItem.attr('size')+'/>');
        }

        //if not make it an h1
        else{
          tmpElem = $('<h1/>');
        }

        //add to the formRow
        formRow.append(elementBuilder(tmpElem,currItem));
      }
      //hidden element for form disable warning if not instructor
      /*else if(currItem.prop('tagName') == "disabledText"){
       //Check if header size is defined and in range
       if(currItem.attr('size') && currItem.attr('size') >= 1 && currItem.attr('size') <= 6){
       tmpElem = $('<h'+currItem.attr('size')+' id="disabledText"/>');
       }
       //if not make it an h1
       else{
       tmpElem = $('<h1 id="disabledText"/>');
       }
       formRow.append(elementBuilder(tmpElem,currItem));
       }*/
      //<header><description> or non-supported element
      else if(currItem.text() != ""){
        tmpElem = $('<p id="'+currItem.prop('tagName')+'"/>');

        //add to the formRow
        formRow.append(elementBuilder(tmpElem,currItem));
      }
    }
    //add the element
    element.append(formRow);

    return element;
  }

  /**
   * Parser for element data
   *
   * @private
   */
  function elementParser(element,data){
    var tmpElem,
      currLabel,
      currItem,
      itemCount = 0;

    for(var i = 0; i < data.length; i++){
      var currItem  = $(data[i]),
        formRow   = currItem.attr('type') != "indyStudy" ? $('<div/>').addClass('rowElem' ) : $('<div/>').addClass('noDisElem');


      //all form elements need a name
      if(!currItem.attr('name')){
        $.error("Form element "+currItem.attr('type')+" must have a name");
      }

      //figure out which element we're dealing with
      //select
      if(currItem.attr('type') == "select"){
        tmpElem = $('<select style="width: auto; height: auto" aria-labelledby="' + currItem[0].id + '_label" />');
        tmpElem = elementBuilder(tmpElem,currItem);
        tmpElem = optionBuilder(tmpElem,currItem.children());
      }
      //radio and check boxes
      else if(currItem.attr('type') == "radio" || currItem.attr('type') == "checkbox"){
        tmpElem = $('<'+currItem.attr('container')+'>');

        //set the container values
        if(currItem.attr('id'))
          tmpElem.attr('id',currItem.attr('id'));
        if(currItem.attr('class'))
          tmpElem.attr('class',currItem.attr('class'));

        tmpElem = radioCheckBuilder(tmpElem,currItem.children('items'),currItem.attr('name'));
      }
      //colorSlider
      else if (currItem.attr('type') == "colorSlider"){
        tmpElem = $('<div />');

        //set the container values
        if(currItem.attr('id'))
          tmpElem.attr('id',currItem.attr('id'));
        if(currItem.attr('class'))
          tmpElem.attr('class',currItem.attr('class'));

        tmpElem = colorSliderBuilder(tmpElem,currItem);
      }
      else if(currItem.attr('type') == "indyStudy"){
        tmpElem = $('<div id="indyStudyHolder" tabindex="0"/>');
        tmpElem.append($('<input type="checkbox" name="indyStudy" tabindex="0" id="indyStudy"/>'));
        tmpElem.append($('<span>'+currItem.attr('label')+'</span>'));
        tmpElem = elementBuilder(tmpElem,currItem);
      }
      //other
      else{
        if(currItem.attr('type') !== "button") {
          tmpElem = $('<input/>').attr('type',currItem.attr('type'));
        }else{
          tmpElem = $('<'+currItem.attr('type')+'/>');
        }
        tmpElem = elementBuilder(tmpElem,currItem);
      }

      //add the label to the form element
      if(Boolean(currItem.attr('label'))){
        if(currItem.attr('type') != "indyStudy"){
          var hiddenAttr = platform.android ? ' aria-hidden="true"' : '';

          formRow.append($('<label' + hiddenAttr + '/>').attr('for',currItem.attr('name')).html((i+1)+". "+currItem.attr('label')));
        }
      }

      formRow.append(tmpElem);

      //add a clear br to submit since it floats right
      if(currItem.attr('type') == 'submit'){
        formRow.append('<br class="clear"/>');
      }

      //add the element
      element.append(formRow);
    }

    return tmpElem;
  }

  /**
   * Builds a color slider element
   * @param element
   * @param data
   * @return {*}
   */
  function colorSliderBuilder(element,data) {
    var min = parseInt(data.attr("min")),
      max = parseInt(data.attr("max")),
      value = parseInt(data.attr("value")),
      name = data.attr("name");

    var sliderElem = $('<div />');
    var squareElem = $('<div />').css({
      "background": "rgb("+value+","+value+","+value+")",
      "border": "3px solid #466A94",
      "width": 50,
      "height": 50,
      "margin-bottom": 20
    });
    var hiddenInput = $('<input />', {
      "type": "hidden",
      "name": name
    });

    _CommonAll.makeSlider(sliderElem,min,max,value,squareElem, hiddenInput);
    element.append(squareElem).append(sliderElem).append(hiddenInput);
    return element;
  }

  /**
   * Builds the element
   *
   * @private
   */
  function elementBuilder(element,data){
    //Got a name, id, class or default val? Add them on
    var defaults = ['name','id','class','value',
      'size','maxlength'];

    for(var i = 0; i < defaults.length; i++){
      if(data.attr(defaults[i]))
        element.attr(defaults[i],data.attr(defaults[i]));
    }

    if(data.attr('format') == "integer" && Boolean(data.attr('min')) && Boolean(data.attr('max'))){
      _CommonAll.checkNumeric(element,data.attr('min'),data.attr('max'),true,data.attr('evenError'),data.attr('boundsError'));
    }

    //buttons need type set
    if(data.attr('type') == "button" || data.attr('type') == "reset" || data.attr('type') == "submit"){
      if (data.attr("type") !== element.attr("type")) {
        element.attr('type',data.attr('type'));
      }
    }
    //add the button text from the label
    if(data.attr('type') == "button"){
      element.html(data.attr('label'));
    }

    //check if I need to make it hidden
    if(data.attr('hidden'))
      element.addClass("hidden");

    //add the text
    if(!Boolean(element.children().length) && data.attr('type') != "button")
      element.html(data.text());

    return element;
  }

  /**
   * build options
   *
   * @private
   */
  function optionBuilder(element,data){
    var options = data.children(),
      tmpOption;

    for(var i = 0; i < options.length; i++){
      var option = $(options[i]);
      tmpOption = $('<option/>');

      //figure out if selected and loaded with a value
      if(option.attr('value'))
        tmpOption.attr('value',option.attr('value'));
      if(option.attr('selected'))
        tmpOption.attr('selected','selected');
      //add it to the select
      element.append(tmpOption.html(option.text()));
    }

    return element;
  }

  /**
   * build radio button and checkboxes
   *
   * @private
   */
  function radioCheckBuilder(element,data,groupName){
    var checks = data.children(); //holds all the checks from data

    for(var i = 0; i < checks.length; i++){
      //apply jQuery to current check
      var check = $(checks[i]);
      var tmpCheck = $('<input type="'+check.prop('tagName')+'"/>'); //holds the current check DOM element $('<span type="'+check.prop('tagName')+'"/>');

      //error out if the elements don't have a name
      if(!Boolean(check.attr('id'))){
        $.error("Checkboxes and radio buttons require a id for labels");
      }

      //figure out if selected and loaded with a value
      if(check.attr('value'))
        tmpCheck.attr('value',check.attr('value'));
      /*if(check.attr('checked'))
       tmpCheck.attr('checked','checked');*/

      //set the id
      tmpCheck.attr('id',check.attr('id'));

      //set the name
      tmpCheck.attr('name',groupName);

      var radioSpan = $('<span/>').addClass('radioCheckSpan');
      var checkLabel = $('<label/>').addClass('radioCheckLabel')
        .attr('for',check.attr('id'))
        .html(check.text());


      radioSpan.append(tmpCheck);
      radioSpan.append(checkLabel);

      element.append(radioSpan);
    }
    return element;
  }
};

/**
 * Build a data object from values in XML file
 * Very useful for large data sets
 */
AdvancedSettingsParser.prototype.buildDataObject = function(){
  var tmpDataObject = this._xml.children('data');

  //loop the data elements
  for(var i = 0; i <  tmpDataObject.length; i++){
    var currData = $(tmpDataObject[i]);
    var tmpDataObjectChildren = currData.children();
    this._dataObject[currData.attr("name")] = {};

    //Now loop all sets and items
    for(var j = 0; j <  tmpDataObjectChildren.length; j++){
      crawlDataXML(tmpDataObjectChildren[j],this._dataObject[currData.attr("name")]);
    }
  }

  //return the data through the function param
  return this._dataObject;

  //recursive function to crawl the data xml
  function crawlDataXML(DataXML,parentObj){
    //make it a jQuery object
    DataXML = $(DataXML);

    for(var i = 0; i < DataXML.length; i++){
      //if set create a new object, call itself
      if(DataXML[i].tagName.toLowerCase() === "set"){
        parentObj[$(DataXML[i]).attr("name")] = {};
        crawlDataXML($(DataXML[i]).children(),parentObj[$(DataXML[i]).attr("name")]);
      }
      //if item loop the items
      else if(DataXML[i].tagName.toLowerCase() === "item"){
        parentObj[i] = $(DataXML[i])[0].textContent;
      }
    }
  }
};