/**
 * @fileoverview Handles loading of experiment datasets from experiments.xml
 * Called from factory.js
 * 
 * @todo Get this running of a base object for common methods like setData 
 */

/**
 * Set us up the ExperimentParser object
 * 
 * @param {object} data (required)      - XML data to set
 * @param {string} container (optional) - ID of the container for html output 
 * @constructor
 */
var ExperimentParser = function(data,container){
	this._data       = undefined;
	this._dataObject = {};
	this._xml        = undefined;
	this._container  = 'experiment';
	
	//Set the data
	this.setData(data);
	
	//Set the container
	this.setContainer(container);
};

/**
 * Load the object with a new XML object
 * 
 * @param {object} data (required) - New XML data to set
 */
ExperimentParser.prototype.setData = function(data){
	//Set data and xml object
	if(data){
		this._data       = data;
		this._xml        = $(this._data);
	}
	else{
		//@todo exception on failure
		alert('ExperimentParser.setData needs data');
	}
};

/**
 * Set the html container for drawing HTML from XML
 * @todo Make this part of a base object
 * 
 * @param {string} container (required) - ID of the container
 */
ExperimentParser.prototype.setContainer = function(container){
	/**
	 * set up the container for HTML output. If it doesn't exist we use the
	 * default for the document
	 */ 
	if(container && typeof container === "string"){
		this._container = $('#'+container);
	}
	else{
		//@todo Could throw an exception here
		this._container = $('#'+this._container);
	}
};

/**
 * Build and display html from the XML data
 * Not used for experiments currently, is here for consistency
 */
ExperimentParser.prototype.buildHTML = function(){
};

/**
 * Build a data object from values in XML file
 * Very useful for large data sets
 */
ExperimentParser.prototype.buildDataObject = function(){
	var tmpDataObject = this._xml.children('datasets').children('data');
	
	//loop the data elements
	for(var i = 0; i <  tmpDataObject.length; i++){
		var currData = $(tmpDataObject[i]);
		var tmpDataObjectChildren = currData.children();
		this._dataObject[currData.attr("name")] = {};
		
		//Now loop all sets and items
		for(var j = 0; j <  tmpDataObjectChildren.length; j++){
			crawlDataXML(tmpDataObjectChildren[j],this._dataObject[currData.attr("name")]);
		}
	}

	//return the data through the function param
	return this._dataObject;
	
	//recursive function to crawl the data xml
	function crawlDataXML(DataXML,parentObj){
		//make it a jQuery object
		DataXML = $(DataXML);
		
		for(var i = 0; i < DataXML.length; i++){
			//if set create a new object, call itself 
			if(DataXML[i].tagName.toLowerCase() === "set"){
				parentObj[$(DataXML[i]).attr("name")] = {};
				crawlDataXML($(DataXML[i]).children(),parentObj[$(DataXML[i]).attr("name")]);
			}
			//if item loop the items
			else if(DataXML[i].tagName.toLowerCase() === "item"){
				if(!Boolean($(DataXML[i]).attr('name'))){
					//gets IE to behave, looks nasty
					if(DataXML.attr('type') == "int"){
						parentObj[i] = parseInt($($(DataXML[i])[0]).text());
					}
					else if(DataXML.attr('type') == "float"){
						parentObj[i] = parseFloat($($(DataXML[i])[0]).text());
					}
					else{
						parentObj[i] = $($(DataXML[i])[0]).text();
					}
				}
				else{
					//gets IE to behave, looks nasty
					if(DataXML.attr('type') == "int"){
						parentObj[$(DataXML[i]).attr('name')] = parseInt($($(DataXML[i])[0]).text());
					}
					else if(DataXML.attr('type') == "float"){
						parentObj[$(DataXML[i]).attr('name')] = parseFloat($($(DataXML[i])[0]).text());
					}
					else{
						parentObj[$(DataXML[i]).attr('name')] = $($(DataXML[i])[0]).text();
					}
				}
			}
		}
	}
};