(function() {
  'use strict';

  Array.prototype.shuffle = function () {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function () {
    this.trialNum = 4;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.names = [];
    this.results = {
      'self': 0,
      'other': 0
    };
  };

  Trial.prototype.init = function () {
    this.createData();
    $(document).ready(function () {
      $('.pause-text-1').focus();
      $('body').prepend($('<div class="ada-shadow-speech" aria-live="polite"></div>'));
      
      if (window.opener.platform.os.family === 'iOS') {
        $('body').prepend($('<span id="focus_elem" tabindex="0" class="visually-hidden">Experiment Tab</span>'));
        setTimeout(function () {
          $('#focus_elem').focus();
        }, 2000);
      }
      if (window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android') {
        if (window.opener.platform.os.family === 'iOS') {
          $('*:not(button)').attr('tabindex', 0);
          $('*').removeAttr('aria-label');
        }
        if (window.opener.platform.os.family === 'Android') {
          for (var i = 0; i < 2; i++) {
            $('.pause-text-' + (i + 1)).attr('aria-label', $('.pause-text-' + (i + 1)).attr('aria-label').replace('Press the space bar to begin the experiment.', 'Press the space bar or double tap to begin the experiment.'));
          }
        }
        $('br').remove();

        var buttonsStr = '';
        var startNum = 0;
        var endNum = 10;

        for (var i = startNum; i < endNum; i++) {
          buttonsStr += '<button id="button_' + i + '">' + i + '</button>';
        }

        $('.experiment').append($('<div class="button_container">' + buttonsStr + '</div>'));

        $('.button_container button').click(function () {
          var answer = +$(this).text();
          trial.addAnswer(answer);
          if ( trial.i === 2) {
            trial.pause();
            trial.trialTextArray[trial.i].hide();
            if (trial.isMobile) {
              $('.trial-text').hide();
            }
          }
          else {
            trial.showTrial();
            if (trial.isMobile) {
              $('.trial-text').show();
            }
          }
        });

        $('.button_container').hide();

        $('.press_instruction').html('Press the space bar or <span class="visually-hidden">double</span> tap<br/>to begin the experiment.');

        for (var i = 0; i < 5; i++) {
          setTimeout(function () {
            $('.experiment').focus();
          }, 100);
        }
        $('.experiment').bind('click', function () {
          trial.start('begin');
          $('.experiment').unbind('click');
        });
      }
    });
    document.onkeypress = this.startKeyListener.bind( this, 'begin' );
  };

  Trial.prototype.startKeyListener = function ( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    var str = String.fromCharCode( keyCode );
    if ( str === ' ' ) {
      this.start( mode );
    }
  };

  Trial.prototype.setAriaFocusTo = function ($elem, delay) {
    delay = delay || 0;

    setTimeout(function () {
      $elem = typeof $elem === 'string' ? $($elem) : $elem;

      if (this.isIos && $elem.find('*').length) {
        $elem.find('.ios-vo-focus-fix').remove();
        // $elem.find('.visible-part').remove();

        $elem.css('position', 'relative');
        var $elemText = $elem.attr('aria-label') || $elem.text();
        var itemDescribedByText = $('#' + $elem.attr('aria-describedby')).text() || '';
        var $iosVoFocusableElem = $('<div class="ios-vo-focus-fix" tabindex="0">' + $elemText + ' ' + itemDescribedByText + '</div>');
        if (!$elem.find('.visible-part').length) {
          $elem.html('<div aria-hidden="true" class="visible-part">' + $elem.html() + '</div>');
        }
        $elem.prepend($iosVoFocusableElem);
        $elem = $iosVoFocusableElem;
      }

      $elem.focus();
    }.bind(this), delay);
  };

  Trial.prototype.createData = function () {
    for ( var i = 0; i < 4; i++ ) {
      this.data.push({ 'new': true });
    }
    this.data.shuffle();
  };

  Trial.prototype.pause = function () {
    this.trialTextArray[4].hide();
    this.$trialCounter.hide();
    this.$liveRegion.html('');

    this.$pauseText2.show().focus();
    this.$textContainer.show();

    if (this.isMobile) {
      this.$pauseText2.find('[tabindex="0"]').focus();
      this.$btnContainer.hide();

      $('.exp2').bind('click', function () {
        trial.start('after pause');
        $('.exp2').unbind('click');
      });
    }

    if (this.isIos) {
      this.setAriaFocusTo(this.$pauseText2);
      this.$liveRegion.html(this.$pauseText2.text());
      setTimeout(function() {
        this.$liveRegion.html('');
      }.bind(this), 14000);
    }
    document.onkeypress = this.startKeyListener.bind( this, 'after pause' );
  };

  Trial.prototype.start = function ( mode ) {
    if ( mode === 'begin' ) {
      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounter = $( '.trial-counter' );
      this.trialTextArray = [];
      this.$pauseText1 = $( '.pause-text-1' );
      this.$pauseText2 = $( '.pause-text-2' );
      this.$textContainer = $( '.text-container' );
      this.$container = $( '.application-container' );
      this.$btnContainer = $('.button_container');
      this.isMobile = window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android';
      this.isIos = window.opener.platform.os.family === 'iOS';
      this.isAndroid = window.opener.platform.os.family === 'Android';
      this.$closeButton = $( '.close-button' );
      this.$membersAnswers = $( '.members-answers' );
      this.$liveRegion = $( '.ada-shadow-speech' );
      for ( var i = 1; i <= 4; i++ ) {
        this.trialTextArray[i] = $( '.trial-text-'+i );
      }
    }
    else {
      this.$membersAnswers.show();
    }
    if (this.isMobile) {
      this.$btnContainer.show();
      $('.trial-text').append($('.trial-p').find('em').eq(0));
      $('.trial-p').find('em').remove();
      for ( var i = 1; i <= 4; i++ ) {
        $('.trial-text-' + i).html($('.trial-text-' + i).text());
      }
    }
    this.$pauseText1.hide();
    this.trialTextArray[this.i + 1].show();
    this.$trialCounter.show();
    this.$textContainer.hide();
    this.$pauseText2.hide();

    document.onkeypress = this.keyListener.bind( this );
    this.showTrial();
  };

  Trial.prototype.saveResult = function () {
    this.$trialCounter.hide();
    this.trialTextArray[4].hide();
    this.$membersAnswers.hide();
    if (this.isMobile) {
      this.$btnContainer.hide();
      $('.trial-text').hide();
      $('em').hide();
    }
    this.$closeButton.show();
    this.$closeButton.find('button').focus();

    for ( var i = 0; i < 2; i++ ) {
      this.results['self'] += this.answers[i];
    }
    this.results['self'] = this.results['self'] / 2;
    for ( i = 2; i < 4; i++ ) {
      this.results['other'] += this.answers[i];
    }
    this.results['other'] = this.results['other'] / 2;
    window.results = this.results;
  };

  Trial.prototype.showTrial = function () {
    if ( this.i < this.trialNum ) {
      var html = '';
      this.$membersAnswers.html( html );
      this.i++;
      this.$trialCounterCurrent.text( this.i );

      if (this.i >= 3) {
        $('.trial-text').show();
      }
      this.trialTextArray[this.i].show().focus();
      if (this.isIos) {
        setTimeout(function() {
          this.$liveRegion.html(this.$trialCounter.text() + '. ' + this.trialTextArray[this.i].text());
        }.bind(this), 20);
      }
      else {
        this.$liveRegion.html(this.$trialCounter.text());
      }
      if (this.isAndroid) {
        setTimeout(function () {
          this.trialTextArray[this.i].focus();
        }.bind(this), 20);
      }
      if (this.i !== 1) {
        this.trialTextArray[this.i-1].hide();
      }
    }
    else {
      this.saveResult();
    }
  };

  Trial.prototype.addAnswer = function ( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function ( event ) {
    var keyCode = event.keyCode || event.which;
    var str;

    str = String.fromCharCode( keyCode );

    if ( '99887766554433221100'.indexOf( str ) > -1 ) {
      ( str === '9' || str === '9' ) && ( this.addAnswer( 9 ) );
      ( str === '8' || str === '8' ) && ( this.addAnswer( 8 ) );
      ( str === '7' || str === '7' ) && ( this.addAnswer( 7 ) );
      ( str === '6' || str === '6' ) && ( this.addAnswer( 6 ) );
      ( str === '5' || str === '5' ) && ( this.addAnswer( 5 ) );
      ( str === '4' || str === '4' ) && ( this.addAnswer( 4 ) );
      ( str === '3' || str === '3' ) && ( this.addAnswer( 3 ) );
      ( str === '2' || str === '2' ) && ( this.addAnswer( 2 ) );
      ( str === '1' || str === '1' ) && ( this.addAnswer( 1 ) );
      ( str === '0' || str === '0' ) && ( this.addAnswer( 0 ) );
      if ( this.i === 2) {
        this.pause();
        this.trialTextArray[this.i].hide();
        if (this.isMobile) {
          $('.trial-text').hide();
        }
      }
      else {
        this.showTrial();
        if (this.isMobile) {
          $('.trial-text').show();
        }
      }
    }
  };

  var trial = new Trial();

  trial.init();
})();
