(function() {
  'use strict';

  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 3;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.names = [];
    this.results = {
      'practice trial': 0,
      'unanimous group': 0,
      'dissenter present': 0
    };


  };

  Trial.prototype.init = function() {
    this.createData();
    window.onkeypress = this.startKeyListener.bind( this, 'begin' );
    $(document).ready(function () {
      // alert(window.opener.platform.os.family);
      if (!window.platform.mobile) {
        $('.text-container-1, .text-container-2').keyup(function (e) {
          trial.startKeyListener('begin', e);
        });
      }
      if (window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android') {
        $('.experiment').append($('<div class="button_container"><button tabindex="0" aria-label="Yes">Yes</button><button tabindex="0" aria-label="No">No</button>'));
        $('.button_container button').click(function () {
          var answer = (this.innerText === 'Yes' ? 1 : 0);
          trial.addAnswer(answer)
          if ( trial.i === 3 ) {
           trial.trialTextArray[trial.i].hide();
          }
          trial.showTrial();
        });
        $('.button_container').hide();

        $('#press_instruction').html('Press the space bar or <span class="visually-hidden">double</span> tap <br/>to begin the experiment.');
        // $('.text-container-2').attr('aria-label', $('.text-container-2').attr('aria-label').replace(' Press the space bar to begin the experiment.', ' Double tap on screen to begin the experiment.'));
        $('.trial-text').html($('.trial-text').html().split('(Press y or n)').join('') + '<br/><span class="keyboard_instruction">Select y or n on your keyboard or use the buttons on the screen if you don\'t have a keyboard.</span>');
        $('.keyboard_instruction').hide();

        for (var i = 0; i < 3; i++) {
          setTimeout(function () {
            $('.experiment').focus();
          }, 100);
        }
        $('.experiment').bind("click", function () {
          trial.start('begin');
          $('.experiment').unbind('click');
        });
      }
      if (window.platform.IE) {
        var splittedText = $('.experiment > p').text().split('women can cook."');
        $('.experiment > p').attr('aria-label', splittedText[0]);
        $('#hidden_p').text('women can cook."' + splittedText[1]);
      }
      else {
        $('#hidden_p').text($('.experiment > p').text());
      }

      $('.close-button button').click(function (e) {
        e.preventDefault();
        window.opener.player.close_experiment();
      });
      $('.close-button button').keyup(function (e) {
        if (e.keyCode === 32) {
          e.preventDefault();
          if (window.platform.iOS) {
            window.focus();
            window.close();
          }
          else {
            window.opener.player.close_experiment();
          }
        }
      });

      if (window.opener.platform.os.family === 'iOS') {
        $('.experiment').prepend('<span id="expFocus" tabindex="0" class="visually-hidden">Experiment. Swipe the screen to read the instruction.</span>');
        setTimeout(function () {
          // $('#press_instruction').attr('tabIndex', '0');
          for (var i = 0; i < 300; i++) {
            setTimeout(function () {
              $('#expFocus').focus();
            }, 100);
          }
        }, 3000);
        $('body').append($('<div class="ada-shadow-speech" aria-live="polite"></div>'));
        // alert(window.opener.platform.os.family);
      }
    });
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    var str = String.fromCharCode( keyCode );
    if ( str === ' ' ) {
      this.start( mode );
    }
  };

  Trial.prototype.createData = function() {
    for ( var i = 0; i < 3; i++ ) {
      this.data.push( { 'new': true } );
    }
    this.data.shuffle();
  };

  Trial.prototype.pause = function() {
    this.trialTextArray[3].hide();
    this.$trialCounter.hide();
    this.$image.hide();

    this.$pauseText.show();
    this.$textContainer.show();
    this.$textContainer2.show();

    window.onkeypress = this.startKeyListener.bind( this, 'after pause' );
  };

  Trial.prototype.start = function( mode ) {
    if ( mode === 'begin' ) {
      $('.experiment > p').attr('tabIndex', '-1').removeAttr('aria-describedby');
      $('#hidden_p').remove();
      this.$keyboardInstruction = $('.keyboard_instruction');
      this.$pressInstruction = $('#press_instruction');
      this.$button_container = $('.button_container');
      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounter = $( '.trial-counter' );
      this.trialTextArray = [];
      this.$image = $( '.photo' );
      this.$pauseText = $( '.pause-text' );
      this.$textContainer = $( '.text-container-1' );
      this.$textContainer2 = $( '.text-container-2' );
      this.$container = $( '.application-container' );
      this.$closeButton = $( '.close-button' );
      this.$membersAnswers = $( '.members-answers' );
      for ( var i = 1; i <= 3; i++ ) {
        this.trialTextArray[i] = $( '.trial-text-' + i );
      }
      this.$button_container.show();
    }
    else {
      this.$membersAnswers.show();
    }
    this.$pressInstruction.attr('tabIndex', '-1').hide();
    this.$pauseText.attr('tabIndex', '-1').hide();
    this.trialTextArray[this.i + 1].attr('tabIndex', '0').show();
    this.$keyboardInstruction.show();
    this.$trialCounter.show();
    this.$textContainer.attr('tabIndex', '-1').hide();
    this.$textContainer2.attr('tabIndex', '-1').hide();

    if (!platform.mobile) {  
      window.onkeypress = this.keyListener.bind( this );
    }
    this.showTrial();
  };

  Trial.prototype.saveResult = function() {
    this.$trialCounter.hide();
    this.$keyboardInstruction.hide();
    this.trialTextArray[3].hide();
    this.$membersAnswers.hide();

    window.results=[];

    this.$closeButton.show();
    $('.experiment > p').hide();
    for ( var i = 0; i < 3; i++ ) {
      window.results.push(this.answers[i]);
      window.results.push(1);
    }
    for (var i = 0; i < 3; i++) {
      setTimeout(function () {
        trial.$closeButton.find('button').focus();
      }, 100)
    }
  };

  Trial.prototype.showTrial = function() {
    if ( this.i < this.trialNum ) {
      var html = '';
      this.$membersAnswers.html( html );
      this.i++;
      this.$trialCounterCurrent.text( this.i );
      if (this.trialTextArray[this.i - 1]) {
        this.trialTextArray[this.i - 1].hide();
      }
      this.trialTextArray[this.i].attr('tabIndex', '0').show();
      if (this.trialTextArray[this.i - 1]) {
        this.trialTextArray[this.i - 1].attr('tabIndex', '-1').removeAttr('aria-label');
      }
      if (window.opener.platform.os.family === 'iOS') {
        this.trialTextArray[this.i].removeAttr('aria-label');
        $('.ada-shadow-speech').html();
        $('.ada-shadow-speech').html('Trial ' + this.i + ' of ' + this.trialNum + this.trialTextArray[this.i].text());
      }
      else {
        this.trialTextArray[this.i].attr('aria-label', 'Trial ' + this.i + ' of ' + this.trialNum + '. ' + this.trialTextArray[this.i].text());
      }
      this.trialTextArray[this.i].focus(); 
    }
    else {
      this.saveResult();
      this.$button_container.hide();
      this.trialTextArray[this.i].attr('tabIndex', '-1');
      this.$closeButton.find('button').attr('tabIndex', '0').focus();
    }
  };

  Trial.prototype.addAnswer = function( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function( event ) {
    var keyCode = event.keyCode || event.which;
    var str;

    str = String.fromCharCode( keyCode );

    if ( 'yYnN'.indexOf( str ) > -1 ) {
      ( str === 'y' || str === 'Y' ) && ( this.addAnswer( 1 ) );
      ( str === 'n' || str === 'N' ) && ( this.addAnswer( 0 ) );
      if ( this.i === 3 ) {
        this.trialTextArray[this.i].hide();
      }
      this.showTrial();
    }
  };

  var trial = new Trial();

  trial.init();

})();