(function() {
  'use strict';

  var status = false,
      sex = 'male',
      BLUE_BACKGROUND = 'linear-gradient(to bottom, #5DBFE8 0%, #599BC4 100%)';

  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 10;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.names = [];
    this.resultsMale = {};
    this.resultsFemale = {};

    this.isInitialized = false;
    this.isSecondExperimentPromoShown = false;
    this.platform = window.opener.platform;
    this.platformFamily = this.platform.os.family;
    this.isIos = this.platformFamily === 'iOS';
    this.isAndroid = this.platformFamily === 'Android';
    this.isTablets = this.isIos || this.isAndroid;
    this.isIE = this.platform.name === 'IE';
    this.isMacSafari = this.platform.name === 'Safari' && !this.isIos;
    this.isFF = this.platform.name === 'Firefox';

    this.tapOrPressInstructionPart = this.isTablets ? 'Press the space bar or <span class="visually-hidden">double</span> tap' : 'Press the space bar';
  };

  Trial.prototype.init = function() {
    if (this.isTablets) {
      window.onclick = this.startKeyListener.bind( this, 'begin' );
    }
    else {
      window.onkeypress = this.startKeyListener.bind( this, 'begin' );
    }

    if (this.isTablets) {
      this.tabletsInit();
    }
    var iosInitFocusDealy = this.isIos ? 1100 : 0;
    this.setAriaFocusTo('.text-container', iosInitFocusDealy);
  };

  Trial.prototype.tabletsInit = function () {
    setTimeout(function () {
      var $devicesTypeInstructions = $('.devices-type-instruction');
      $devicesTypeInstructions.html(this.tapOrPressInstructionPart);
      $devicesTypeInstructions.parent().removeAttr('aria-label');

      $('#textarea-label').attr({
        'aria-hidden': true,
        'role': 'presentation'
      });

      this.$blueContainer = $('.blue-container');
    }.bind(this), 0);
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    var str = evt.type === 'keypress' ? String.fromCharCode( keyCode ) : ' ';
    if ( str === ' ') {
      this.start( mode );
    }
  };


  Trial.prototype.pause = function() {
    this.$brieflyContainer.hide();

    //this.trialTextArray[10].hide();
    //this.$trialCounter.hide();
    //this.$image.hide();
    this.$pauseText.show();
    this.$pauseText.html('Next we would like you to rate your teacher on a number of domains.');
    this.$textContainer.attr('aria-describedby', this.$pauseText.attr('id')).html(this.tapOrPressInstructionPart + ' <br/>to continue');

    var validTextContainerText = !this.isIos ?
                                 this.$textContainer.text() :
      // this.$textContainer.html().replace(/<div class="ios-vo-focus-fix".*?>.*?<\/div>|<.*?>/ig, '');
                                 this.$textContainer.find('.visible-part').text() || this.$textContainer.text();

    this.$textContainer.attr('aria-label', validTextContainerText).show();

    // JAWS don't read aria-described
    if (this.isIE ){
      this.$textContainer.attr('aria-label', this.$textContainer.attr('aria-label') + ' ' +  $('#instruction-text').text());
    }

    this.setAriaFocusTo(this.$textContainer);

    if (this.isTablets) {
      window.onclick = this.startKeyListener.bind( this, 'after pause' );
    }
    else {
      window.onkeypress = this.startKeyListener.bind( this, 'after pause' );
    }
  };

  Trial.prototype.start = function( mode ) {

    if ( mode === 'begin') {
      this.isInitialized = true;

      if (this.isFF) {
        $('#textarea-label').removeAttr('role');
      }

      $( '.briefly-inner button' ).click(function(e){
        e.stopPropagation();

        var $textarea = $( '.briefly-inner textarea' );

        if($textarea.val() != ''){
          status = true;

          var $warningMessage = $('.error');

          if ($warningMessage.length) {
            $warningMessage.remove();
            $textarea.attr('aria-labelledby', 'textarea-label');
          }

          this.pause();
        } else {
          $('.error' ).remove();
          $textarea.after('<p class="error" id="warning-message" style="margin-left: 10px; font-style: italic; font-size: 12px;">You must enter an answer.</p>');
          $textarea.attr('aria-labelledby', 'warning-message textarea-label');
          this.setAriaFocusTo($textarea);
        }
      }.bind(this));

      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounter = $( '.trial-counter' );
      this.trialTextArray = [];
      this.$brieflyContainer = $( '.briefly-container' );
      this.$brieflyStrong = $( '.briefly-container strong' );
      this.$brieflyTextarea = $( '.briefly-inner textarea' );
      this.$image = $( '.photo' );
      this.$pauseText = $( '.pause-text' );
      this.$textContainer = $( '.text-container' );
      this.$textContainerTop = $( '.text-container-top' );
      this.$closeButton = $( '.close-button' );
      this.$membersAnswers = $( '.members-answers' );
      this.$tabletsButtonsContainer = $('.tablets-button-container');
      this.$tabletsChoiceButtons = $('.tablets-button-container button');

      for ( var i = 1; i <= 10; i++ ) {
        this.trialTextArray[i] = $( '.trial-text-' + i );
      }
      this.$brieflyContainer.show();
      this.setAriaFocusTo(this.$brieflyContainer.find('textarea'));
    }
    else {
      //debugger
      this.isSecondExperimentPromoShown = true;
      // $('#instruction').removeAttr('aria-hidden').attr('tabindex', 0);        <--------------------------------
      this.$membersAnswers.show();
      this.trialTextArray[this.i + 1].show();
      this.$textContainerTop.show();
      this.$trialCounter.show();

      if (this.isTablets) {
        this.$tabletsButtonsContainer.show();
        //this.$tabletsChoiceButtons.click(this.keyListener.bind( this ));
        this.$tabletsChoiceButtons.unbind('click');
        this.$tabletsChoiceButtons.bind('click', this.keyListener.bind( this ));
      }
    }
    this.$pauseText.hide();
    this.$pauseText.removeAttr("tabindex");
    this.$textContainer.removeAttr('aria-describedby');
    this.$textContainer.hide();

    if (this.isTablets) {
      window.onclick = this.keyListener.bind( this );
    }
    else {
      window.onkeypress = this.keyListener.bind( this );
    }
    this.showTrial( mode );
  };

  Trial.prototype.saveResult = function( arg ) {

    this.$trialCounter.hide();
    this.trialTextArray[10].hide();
    this.$membersAnswers.hide();
    this.$textContainerTop.hide();

    if (this.isTablets) {
      this.$tabletsButtonsContainer.hide();
    }

    if(arg === 'male'){
      this.resultsMale['intelligent'] = this.answers[1];
      this.resultsMale['competent'] = this.answers[6];
      this.resultsMale['smart'] = this.answers[8];
      window.results = this.resultsMale;
        console.log(this.resultsMale);
      status = false;
      sex = 'female';

      if (this.isTablets) {
        window.onclick = this.startKeyListener.bind( this, 'begin' );
      }
      else {
        window.onkeypress = this.startKeyListener.bind( this, 'begin' );
      }

      this.$textContainer.html(this.tapOrPressInstructionPart + ' <br/>to continue the experiment.');
      this.$textContainer.show();
      this.$brieflyStrong.html('her');
      this.$brieflyTextarea.val('');
      this.i = 0;
      this.setAriaFocusTo(this.$textContainer);
      this.cleanIOSHiddenFocusElements();

      this.answers = [];
    } else if(arg === 'female') {
      this.resultsFemale['intelligent'] = this.answers[1];
      this.resultsFemale['competent'] = this.answers[6];
      this.resultsFemale['smart'] = this.answers[8];
      window.results[1] = this.resultsFemale;

      console.log(this.resultsFemale);
      this.$closeButton.show();

      this.setAriaFocusTo(this.$closeButton.find('button'));
    }

  };

  Trial.prototype.showTrial = function( mode ) {
    this.notCompletedSentenceText = this.$textContainerTop.eq(1).text();

    if ( mode === 'begin' ) {
      return false;
    }
    else {
     // debugger
      if ( this.i < this.trialNum ) {
        var html = '';
        this.$membersAnswers.html( html );
        this.i++;
        this.$trialCounterCurrent.text( this.i );

        if (this.trialTextArray[this.i - 1] && this.trialTextArray[this.i - 1].is(':visible')) {
          this.trialTextArray[this.i - 1].hide();

          if (this.isIE || this.isMacSafari) {
            this.trialTextArray[this.i - 1].blur();
          }
        }

        this.trialTextArray[this.i].show();
/*
        var validTrialText = !this.isIos ?
                             this.$trialText.text() :                                                             <----------------------------
                             this.$trialText.find('.visible-part').text() || this.$trialText.text(); */

        this.trialTextArray[this.i].attr('aria-label', this.notCompletedSentenceText + ' ' +  this.trialTextArray[this.i].text()
                                                      + ' ' + this.$trialCounter.text());

        if (this.isIE || this.isMacSafari) {
          //this.trialTextArray.blur(); <-------------------------------------------------------------------------------------------------------
        }

        this.setAriaFocusTo(this.trialTextArray[this.i]);
      }
      else {
        this.saveResult( sex );
      }
    }
  };

  Trial.prototype.cleanIOSHiddenFocusElements = function () {
    this.trialTextArray.forEach(function (item) {
      var $iosHiddenFocusElements = item.find('.ios-vo-focus-fix');
      if ($iosHiddenFocusElements.length) {
        $iosHiddenFocusElements.remove();
      }
    });
  };

  Trial.prototype.addAnswer = function( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function( event ) {
    var keyCode = event.keyCode || event.which;
    var str;

    str = event.type === 'keypress' ? String.fromCharCode( keyCode ) : event.currentTarget.innerText;


    if ( '998877665544332211'.indexOf( str ) > -1 && status) {
      ( str === '9' || str === '9' ) && ( this.addAnswer( 9 ) );
      ( str === '8' || str === '8' ) && ( this.addAnswer( 8 ) );
      ( str === '7' || str === '7' ) && ( this.addAnswer( 7 ) );
      ( str === '6' || str === '6' ) && ( this.addAnswer( 6 ) );
      ( str === '5' || str === '5' ) && ( this.addAnswer( 5 ) );
      ( str === '4' || str === '4' ) && ( this.addAnswer( 4 ) );
      ( str === '3' || str === '3' ) && ( this.addAnswer( 3 ) );
      ( str === '2' || str === '2' ) && ( this.addAnswer( 2 ) );
      ( str === '1' || str === '1' ) && ( this.addAnswer( 1 ) );
      this.showTrial();
    }
  };

  Trial.prototype.setAriaFocusTo = function ($elem, delay) {
    delay = delay || 0;

    setTimeout(function () {
      $elem = typeof $elem === 'string' ? $($elem) : $elem;

      if (this.isIos && $elem.find('*').length) {
        $elem.find('.ios-vo-focus-fix').remove();
        // $elem.find('.visible-part').remove();

        $elem.css('position', 'relative');
        var $elemText = $elem.attr('aria-label') || $elem.text();
        var itemDescribedByText = $('#' + $elem.attr('aria-describedby')).text() || '';
        var $iosVoFocusableElem = $('<div class="ios-vo-focus-fix" tabindex="0">' + $elemText + ' ' + itemDescribedByText + '</div>');
        if (!$elem.find('.visible-part').length) {
          $elem.html('<div aria-hidden="true" class="visible-part">' + $elem.html() + '</div>');
        }
        $elem.prepend($iosVoFocusableElem);
        $elem = $iosVoFocusableElem;
      }

      $elem.focus();
    }.bind(this), delay);
  };

  var trial = new Trial();

  trial.init();

})();