(function() {
  'use strict';

  var choiceStudent = ['As mentioned on the previous screen, write around 5 sentences pertaining to why you think the student does so well in school.',
      'As mentioned on the previous screen, write around 5 sentences pertaining to why you think the student does so poorly in school.'],
    trialStudent = ['Sincere', 'Trustworthy', 'Intelligent', 'Affectionate', 'Curious', 'Assertive', 'Hardworking', 'Enthusiastic'];
  var isPassed = true;
  var INSTRUCTION_TEXT = 'In making appraisals of another person, it can also be important to understand the personality of the person making these judgments. For each trait, rate it on a scale of 0 to 9 in terms of how well it describes <strong>you</strong> as a person.';

  //random choice between (well and poorly) students
  var randomChoice = function() {
    return Math.floor( Math.random() * (2 - 1 + 1) );
  };
  var randomNum = randomChoice();


  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 8;
    this.sectionCondition = 1;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.names = [];
    this.results = {
      'self': 0,
      'other': 0
    };
    this.isInitialized = false;
    this.isSecondExperimentPromoShown = false;
    this.platform = window.opener.platform;
    this.platformFamily = this.platform.os.family;
    this.isIos = this.platformFamily === 'iOS';
    this.isAndroid = this.platformFamily === 'Android';
    this.isTablets = this.isIos || this.isAndroid;
    this.isIE = this.platform.name === 'IE';
    this.isMacSafari = this.platform.name === 'Safari' && !this.isIos;
  };

  Trial.prototype.init = function() {
    if (this.isTablets) {
      window.onclick = this.startKeyListener.bind( this, 'begin' );
    }
    else {
      window.onkeypress = this.startKeyListener.bind( this, 'begin' );
    }

    if (this.isTablets) {
      this.tabletsInit();
    }
    var iosInitFocusDealy = this.isIos ? 1100 : 0;
    this.setAriaFocusTo('.text-container', iosInitFocusDealy);
  };

  Trial.prototype.tabletsInit = function () {
    setTimeout(function () {
      var $devicesTypeInstructions = $('.devices-type-instruction');
      $devicesTypeInstructions.html('Press the space bar or <span class="visually-hidden">double</span> tap');
      $devicesTypeInstructions.parent().removeAttr('aria-label');
    }, 0);
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    var str = evt.type === 'keypress' ? String.fromCharCode( keyCode ) : ' ';
    if ( str === ' ' && isPassed) {
      this.start( mode );
    }
  };


  Trial.prototype.pause = function() {
    var $studentChoice = $('.choice-student' );
    $studentChoice.before('<div id="instruction" role="document" aria-labelledby="instruction-text" aria-hidden="true">'
                          + '<span id="instruction-text"">' + INSTRUCTION_TEXT + '</span></div>');
    $studentChoice.parent().css('background-color', 'transparent');
    $studentChoice.remove();
    $('.container-answer' ).remove();

    // fix content for ios
    var validTextContainerText = !this.isIos ?
                                 this.$textContainer.text() :
                                 // this.$textContainer.html().replace(/<div class="ios-vo-focus-fix".*?>.*?<\/div>|<.*?>/ig, '');
                                 this.$textContainer.find('.visible-part').text() || this.$textContainer.text();

    this.$textContainer.attr('aria-label', this.$trialCounter.text() + '. ' + validTextContainerText).show();

    // JAWS don't read aria-descibed
    if (this.isIE ){
      this.$textContainer.attr('aria-label', this.$textContainer.attr('aria-label') + ' ' +  $('#instruction-text').text());
    }


    this.setAriaFocusTo(this.$textContainer);
    this.$trial.show();
    this.$trialCounter.show();

    trial.start('after pause');
    // if (this.isTablets) {
    //   window.onclick = this.startKeyListener.bind( this, 'after pause' );
    // }
    // else {
    //   window.onkeypress = this.startKeyListener.bind( this, 'after pause' );
    // }
  };

  Trial.prototype.start = function( mode ) {

    $('.answer button' ).click(function(e){
      e.stopPropagation();

      var $textarea = $('.answer textarea' );

      if($textarea.val() != ''){
        // trial.start();
        this.pause();
      } else {
        $('.error' ).remove();
        $textarea.after('<p class="error" id="warning-message" style="margin-left: 10px; font-style: italic; font-size: 12px;">You must enter an answer.</p>');
        $textarea.attr('aria-labelledby', 'warning-message textarea-label');
        this.setAriaFocusTo($textarea);
      }
    }.bind(this));

    if ( mode === 'begin' && !this.isInitialized) {
      this.isInitialized = true;

      if ( window.opener.sectionCondition ) {
        this.sectionCondition = window.opener.sectionCondition;
      }
      else if ( window.parent.sectionCondition ) {
        this.sectionCondition = window.parent.sectionCondition;
      }
      window.parent.choiceStudent = randomNum;
      window.opener.choiceStudent = randomNum;
      console.log(randomNum);

      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounter = $( '.trial-counter' );
      this.trialTextArray = [];
      this.$trialStudent = $('.trialStudent');
      this.$trialText = $('.trial-text');
      this.$trial = $('.trial');
      this.$pauseText = $( '.pause-text' );
      this.$textContainer = $( '.text-container' );
      this.$textContainerTop = $( '.text-container-top' );
      this.$closeButton = $( '.close-button' );
      this.$membersAnswers = $( '.members-answers' );
      this.$tabletsButtonsContainer = $('.tablets-button-container');
      this.$tabletsChoiceButtons = $('.tablets-button-container button');

      for ( var i = 0; i < 8; i++ ) {
        this.trialTextArray[i] = $( trialStudent[i] );
      }

      var $choiceContainer = $( '.choice-container' ).show();
      this.setAriaFocusTo($choiceContainer.find('textarea'));

      $( '.choice-student' ).html( choiceStudent[randomNum] );
    }
    else if (mode === 'after pause' && !this.isSecondExperimentPromoShown) {
    //else {
      this.isSecondExperimentPromoShown = true;
      $('#instruction').removeAttr('aria-hidden').attr('tabindex', 0);
      this.$membersAnswers.show();
      this.$trialCounter.show();
      //$( '.choice-container' ).hide();

      this.$trialText.show();

      window.onkeypress = this.keyListener.bind( this );

      if (this.isTablets) {
        this.$tabletsButtonsContainer.show();
        this.$tabletsChoiceButtons.click(this.keyListener.bind( this ));
      }

      this.showTrial(mode);
    }
    //this.$pauseText.hide();
    this.$textContainer.show();
    $( '.text-container' ).hide();


  };

  Trial.prototype.saveResult = function() {

    this.$trialCounter.hide();
    this.$trial.hide();
    this.$membersAnswers.hide();
    this.$textContainerTop.hide();
    this.$closeButton.show();
    $( '.choice-container' ).hide();

    if (this.isTablets) {
      this.$tabletsButtonsContainer.hide();
    }

    this.results = {
      'intelligent': this.answers[2],
      'hardworking': this.answers[6]
    };

    window.results = this.results;

    this.setAriaFocusTo(this.$closeButton.find('button'));
  };

  Trial.prototype.showTrial = function( mode ) {
    if ( mode === 'begin' ) {
      return false;
    }
    else if(isPassed){
      if ( this.i < this.trialNum && isPassed) {
        //this.$trialStudent.html(trialStudent[this.i]+' ');
        $('.trialStudent').html(trialStudent[this.i] + ' ');
        this.$textContainerTop.hide();
        var html = '';
        this.$membersAnswers.html( html );
        this.i++;
        this.$trialCounterCurrent.text( this.i );

        var validTrialText = !this.isIos ?
                                this.$trialText.text() :
                                this.$trialText.find('.visible-part').text() || this.$trialText.text();

        this.$trialText.attr('aria-label', this.$trialCounter.text() + '. ' +  validTrialText);

        if (this.isIE || this.isMacSafari) {
          this.$trialText.blur();
        }
        this.setAriaFocusTo(this.$trialText);

        if(this.i>7){
          this.$textContainer.hide();
          isPassed = false;
        }
      }
    }
  };

  Trial.prototype.addAnswer = function( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function( event ) {
    var keyCode = event.keyCode || event.which;
    var str;

    str = event.type === 'keypress' ? String.fromCharCode( keyCode ) : event.currentTarget.innerText;

    if ( '99887766554433221100'.indexOf( str ) > -1 ) {
      ( str === '9' || str === '9' ) && ( this.addAnswer( 9 ) );
      ( str === '8' || str === '8' ) && ( this.addAnswer( 8 ) );
      ( str === '7' || str === '7' ) && ( this.addAnswer( 7 ) );
      ( str === '6' || str === '6' ) && ( this.addAnswer( 6 ) );
      ( str === '5' || str === '5' ) && ( this.addAnswer( 5 ) );
      ( str === '4' || str === '4' ) && ( this.addAnswer( 4 ) );
      ( str === '3' || str === '3' ) && ( this.addAnswer( 3 ) );
      ( str === '2' || str === '2' ) && ( this.addAnswer( 2 ) );
      ( str === '1' || str === '1' ) && ( this.addAnswer( 1 ) );
      ( str === '0' || str === '0' ) && ( this.addAnswer( 0 ) );
      if ( this.i === 8 ) {
        this.$textContainer.hide();
        this.saveResult();
      }
      else {
        this.showTrial();
      }
    }
  };

  Trial.prototype.setAriaFocusTo = function ($elem, delay) {
    delay = delay || 0;

    setTimeout(function () {
      $elem = typeof $elem === 'string' ? $($elem) : $elem;

      if (this.isIos && $elem.find('*').length) {
        $elem.find('.ios-vo-focus-fix').remove();
       // $elem.find('.visible-part').remove();

        $elem.css('position', 'relative');
        var $elemText = $elem.attr('aria-label') || $elem.text();
        var itemDescribedByText = $('#' + $elem.attr('aria-describedby')).text() || '';
        var $iosVoFocusableElem = $('<div class="ios-vo-focus-fix" tabindex="0">' + $elemText + ' ' + itemDescribedByText + '</div>');
        if (!$elem.find('.visible-part').length) {
          $elem.html('<div aria-hidden="true" class="visible-part">' + $elem.html() + '</div>');
        }
        $elem.prepend($iosVoFocusableElem);
        $elem = $iosVoFocusableElem;
      }

      $elem.focus();
    }.bind(this), delay);
  };

  var trial = new Trial();

  trial.init();

})();