(function() {
  'use strict';

  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 17;
    this.i = 0;
    this.data = [ 'Circle', 'Square', 'Oval', 'Rectangle', 'Octagon', 'Rhombus', 'Triangle' ];
    this.dataSecond = [ 'Lazy', 'Intelligent', 'Talkative', 'Persistent', 'Happy', 'Kind', 'Trustworthy', 'Stubborn', 'Cynical', 'Creative' ];
    this.answers = [];
    this.names = [];
    this.word = Math.floor( Math.random() * 2 ) ? 'Stubborn' : 'Persistent';
  };

  Trial.prototype.init = function() {
    this.data.shuffle();
    setTimeout(function () {
      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounterTotal = $( '.trial-counter .total' );
      this.$trialCounter = $( '.trial-counter' );
      this.$pauseScreen = $( '.pause-screen' );
      this.$closeButton = $( '.close-button' );
      this.$buttonNext = $( '.button-next' );
      this.$trialWord = $( '.first-part .trial-word' );
      this.$startScreen = $( '.start-screen' );
      this.$firstPart = $( '.first-part' );
      this.$secondPart = $( '.second-part' );
      this.$secondPartId = $( '#second-part' );
      this.$preScreen = $( '.pre-screen' );
      this.$startScreenWord = $( '.start-screen .word' );
      this.$secondPartWord = $( '.second-part .trial-word' );
      this.$inputText = $( '.questionText' );
      this.isMobile = window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android';
      this.isIos = window.opener.platform.os.family === 'iOS';
      this.isAndroid = window.opener.platform.os.family === 'Android';

      this.$preScreen.find('.text-container').focus();

      $('body').prepend($('<div class="ada-shadow-speech" aria-live="polite"></div>'));

      if (this.isMobile) {
        if (this.isIos) {
          $('*').attr('tabindex', '0');
        }
        $('[aria-label]').removeAttr('aria-label');
        this.$secondPartId.append($('.second-small'));
        this.$secondPartId.append($('<div class="button_container"></div>'));
        for (var i = 1; i <= 9; i++) {
          var buttonStr = '<button id="button_' + i + '">' + i + '</button>';

          $('.button_container').append($(buttonStr));
        }

        $('.instruction').html( $('.instruction').html().replace('Press the space bar', 'Press the space bar or <span class="visually-hidden">double</span> tap') );

        $('.experiment').bind('click', function () {
          trial.start('pre');
          $('.experiment').unbind('click');
        });

        if (this.isMobile) {
          $('.ada-shadow-speech').html(this.$preScreen.find('.text-container').text());
          this.$preScreen.find('.start_btn').focus();

          if (this.isIos) {
            $('body').prepend($('<span id="focus_elem" tabindex="0" class="visually-hidden">Experiment Tab</span>'));
            setTimeout(function () {
              $('#focus_elem').focus();
            }, 2000);
          }
        }

        $('.button_container').find('button').click(function() {
          trial.addAnswer($(this).html());
          trial.showTrialSecondPart();
        });
      }
    }.bind(this), 0);
    document.onkeypress = this.startKeyListener.bind( this, 'pre' );
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    var str = String.fromCharCode( keyCode );
    if ( str === ' ' ) {
      this.start( mode );
    }
  };

  Trial.prototype.pause = function() {
    this.$firstPart.hide();
    this.$trialCounter.hide();
    this.$pauseScreen.show().focus();

    if (this.isMobile) {
      this.setAriaFocusTo(this.$pauseScreen.find('.instruction'));
      this.liveRegion.html(this.$pauseScreen.text());
    }
    $('.pause-screen').bind('click', function () {
      trial.start('second');
      $('.pause-screen').unbind('click');
    });
    document.onkeypress = this.startKeyListener.bind( this, 'second' );
  };

  Trial.prototype.start = function( mode ) {
    this.liveRegion = $('.ada-shadow-speech');

    if ( mode === 'pre' ) {
      this.$startScreenWord.text( this.word );

      if (!this.isMobile) {
        var ariaLabelStart = this.$startScreen.attr('aria-label').replace('$$word$$', this.word);

        this.$startScreen.attr('aria-label', ariaLabelStart);
      }
      if (window.opener.platform.os.family === 'Android') {
        $('[aria-label]').removeAttr('aria-label');
      }
      this.$preScreen.hide();
      this.$startScreen.show().focus();
      if (this.isMobile) {
        setTimeout(function() {
          this.setAriaFocusTo(this.$startScreen.find('.instruction'));
          this.liveRegion.html(this.$startScreen.text());
        }.bind(this), 0);
      }

      $('.start-screen').bind('click', function () {
        trial.start('first');
        $('.start-screen').unbind('click');
      });
      document.onkeypress = this.startKeyListener.bind( this, 'first' );
    }
    else if ( mode === 'first' ) {
      document.onkeypress = function() {};

      this.$startScreen.hide();
      this.$firstPart.show();
      this.$firstPart.find('.text').eq(0).attr('tabindex', '0').focus();
      this.$trialCounter.show();

      this.liveRegion.html(this.$trialCounter.text());

      this.showTrial();

      this.$buttonNext[0].addEventListener( 'click', this.buttonListener.bind( this ) );
      this.$buttonNext[0].addEventListener( 'keypress', this.buttonListener.bind( this ) );
    }
    else {
      this.$secondPart.show();
      this.$secondPartId.show();
      this.$trialCounterTotal.text( '10' );
      this.$pauseScreen.hide();
      this.$trialCounter.show();

      if (window.opener.platform.name === 'Safari') {
        this.liveRegion.html(this.$secondPart.text());
      }

      this.showTrialSecondPart();

      document.onkeypress = this.keyListener.bind( this );
    }
  };

  Trial.prototype.showTrial = function() {
    this.$trialWord.text( this.data[ this.i ] );
    this.i++;
    this.$trialCounterCurrent.text( this.i );
    this.$inputText.val( '' );

    var liveText = '';

    if (this.isMobile) {
      if (this.i === 1) {
        liveText += $('#fp-inst').text();
      }
      liveText += this.$trialCounter.text() + '. ' + this.$trialWord.text();
    }
    else {
      liveText += this.$trialCounter.text();
    }
    this.liveRegion.html(liveText);
  };

  Trial.prototype.showTrialSecondPart = function() {
    if ( this.i < this.trialNum ) {
      if (!this.isMobile) {
        var ariaLabel = this.$secondPart.attr('aria-label').replace(this.dataSecond[ this.i - 8 ] + '.', this.dataSecond[ this.i - 7 ] + '.');
        this.$secondPart.attr('aria-label', ariaLabel);
      }
      this.$secondPartWord.text( this.dataSecond[ this.i - 7 ] );
      this.i++;
      this.$trialCounterCurrent.text( this.i - 7 );
      this.$secondPartWord.focus();
      if (this.isMobile) {
        this.$secondPartWord.find('.believe').focus();
      }

      if (this.isAndroid) {
        setTimeout(function() {
          this.liveRegion.html(this.$trialCounter.text());
        }.bind(this), 1500);
      } else {
        this.liveRegion.html(this.$trialCounter.text());
      }

      if (window.opener.platform.name === 'Safari') {
        this.liveRegion.html(this.$trialCounter.text() + ' ' + this.$secondPart.text());
      }
      setTimeout(function() {
        this.$secondPart.focus();
      }.bind(this), 100);
    }
    else {
      this.saveResult();
      this.$closeButton.find('button').focus();
    }
  };

  Trial.prototype.saveResult = function() {
    document.onkeypress = function() {};
    this.$secondPart.hide();
    this.$secondPartId.hide();
    this.$trialCounter.hide();

    window.results = {
      'stubborn': this.answers[this.dataSecond.indexOf( 'Stubborn' )],
      'persistent': this.answers[this.dataSecond.indexOf( 'Persistent' )],
      'word': this.word
    };

    this.$closeButton.show();
  };

  Trial.prototype.addAnswer = function( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function( event ) {
    var keyCode = event.keyCode || event.which;
    var str = String.fromCharCode( keyCode );

    if ( str.match( /\d/ ) ) {
      if ( str !== '0' ) {
        this.addAnswer( str );
        this.showTrialSecondPart();
      }
    }
  };

  Trial.prototype.buttonListener = function( e ) {
    if ( e.which === 1 || e.which === 13 ) {
      if ( this.i === 7 ) {
        this.pause();
      }
      else {
        this.showTrial();
        this.$inputText.focus();
      }
    }
  };

  Trial.prototype.setAriaFocusTo = function ($elem, delay) {
    delay = delay || 0;

    setTimeout(function () {
      $elem = typeof $elem === 'string' ? $($elem) : $elem;

      if (this.isIos && $elem.find('*').length) {
        $elem.find('.ios-vo-focus-fix').remove();
        // $elem.find('.visible-part').remove();

        $elem.css('position', 'relative');
        var $elemText = $elem.attr('aria-label') || $elem.text();
        var itemDescribedByText = $('#' + $elem.attr('aria-describedby')).text() || '';
        var $iosVoFocusableElem = $('<div class="ios-vo-focus-fix" tabindex="0">' + $elemText + ' ' + itemDescribedByText + '</div>');
        if (!$elem.find('.visible-part').length) {
          $elem.html('<div aria-hidden="true" class="visible-part">' + $elem.html() + '</div>');
        }
        $elem.prepend($iosVoFocusableElem);
        $elem = $iosVoFocusableElem;
      }

      $elem.focus();
    }.bind(this), delay);
  };

  var trial = new Trial();
  trial.init();
})();
