(function() {
  'use strict';

  var pressPhare = platform.mobile ? 'Press the space bar or double tap to begin the experiment.' : 'Press the space bar to begin the experiment.'

  var instructions = {
    start: 'For each trait listed below, rate it in terms of how well it describes you as a person using the 1 to 9 scale. ' + pressPhare,
    continue: 'On the following screen you will again be making a series of judgments pertaining to traits. For this next part, though, you will be rating the traits in terms of how well they describe most other people. When making these judgments, try to think about people as a whole, not necessarily anyone you know specifically.  For each trait listed below, rate it in terms of how well it describes most other people using the 1 to 9 scale.' + pressPhare
  }

  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 20;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.names = [];
    this.results = {
      'self': 0,
      'other': 0
    };
    this.answered = false;


  };

  Trial.prototype.init = function() {
    var live = platform.android || platform.firefox|| platform.iOS ? 'assertive' : 'polite',
      role = platform.android ? '' : 'role="log"';

    this.liveFeedback =  $('<div id="live-feedback" class="visually-hidden"' + role + ' aria-live="' + live + '"></div>');

    this.createData();

    document.onkeypress = this.startKeyListener.bind( this, 'begin');

    var self = this;

    $(document).ready(function () {
      if (platform.mobile) {
        $('.text-container').html('<br><br><br>Press the space bar or <span class="visually-hidden">double</span> tap<br>to begin the experiment.</div>');
      }

      $('#experiment-container').prepend(self.liveFeedback);

      if (!platform.mobile) {
        $('.experiment').attr('tabindex', '0').focus();
      }

      if (window.platform.iOS) {
        $('.experiment p').eq(0).focus();
        $('body').prepend($('<span id="focus_elem" tabindex="0" class="visually-hidden">Experiment Tab</span>'));
        setTimeout(function () {
          $('#focus_elem').focus();
          self.liveFeedback.html(instructions.start);
        }, 2000);
      }

      if (window.platform.firefox) {
        setTimeout(function () {
          self.liveFeedback.html(instructions.start);
        }, 2000);
      }

      else if (!window.platform.IE) {
        self.liveFeedback.html(instructions.start);
      }

      if (window.platform.iOS || window.platform.android) {
        var buttonsStr = '';
        var startNum = 1;
        var endNum = 10;

        for (var i = startNum; i < endNum; i++) {
          buttonsStr += '<button id="button_' + i + '">' + i + '</button>';
        }

        $('.experiment').append($('<div class="button_container">' + buttonsStr + '</div>'));

        $('.button_container button').click(function (e) {
          e.stopPropagation();
          var answer = $(this).html();
          trial.addAnswer(answer);
          if ( trial.i === 10 ) {
            trial.pause();
          }
          else {
            trial.showTrial();
          }
        });
        $('.button_container').hide();

        $('.experiment').bind('click', function () {
          trial.start('begin');
          $('.experiment').unbind('click');
        });
      }

      $('.close-button button').click(function (e) {
        e.preventDefault();
        window.opener.player.close_experiment();
      });
      $('.close-button button').keyup(function (e) {
        if (e.keyCode === 32) {
          e.preventDefault();
          if (window.platform.iOS) {
            window.focus();
            window.close();
          }
          else {
            window.opener.player.close_experiment();
          }
        }
      });
    });
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    var str = String.fromCharCode( keyCode );
    if ( str === ' ' ) {
      if (window.platform.IE) {
        $('.experiment').removeAttr('tabindex');
      }
      this.start( mode );
    }
  };

  Trial.prototype.createData = function() {
    for ( var i = 0; i < 20; i++ ) {
      this.data.push( { 'new': true } );
    }
    this.data.shuffle();
  };

  Trial.prototype.pause = function() {
    this.trialTextArray[10].hide();
    this.$trialCounter.hide();
    this.$image.hide();

    this.$pauseText.show();
    $('.button_container').hide();
    this.$textContainer.show();
    this.$textContainerTop.hide();
    this.$textContainerBottom.show();
    document.onkeypress = this.startKeyListener.bind( this, 'after pause' );

    if (window.platform.mobile) {
      $('.experiment').bind('click', function () {
        trial.start('begin');
        $('.experiment').unbind('click');
      });
    }

    if (!window.platform.mobile) {
      $('.experiment').focus();
    }

    var self = this;

    if (window.platform.iOS) {
        $('#focus_elem').focus();
    }

    if (window.platform.firefox) {
      setTimeout(function () {
        self.liveFeedback.html(instructions.continue);
      }, 2000);
    }

    else if (!window.platform.IE) {
      self.liveFeedback.html(instructions.continue);
    }
  };

  Trial.prototype.start = function( mode ) {
    if ( mode === 'begin' ) {
      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounter = $( '.trial-counter' );
      this.trialTextArray = [];
      this.$image = $( '.photo' );
      this.$pauseText = $( '.pause-text' );
      this.$textContainer = $( '.text-container' );
      this.$textContainerTop = $( '.text-container-top' );
      this.$textContainerBottom = $( '.text-container-top2' );
      this.$closeButton = $( '.close-button' );
      this.$membersAnswers = $( '.members-answers' );
      for ( var i = 1; i <= 20; i++ ) {
        this.trialTextArray[i] = $( '.trial-text-'+i );
      }
    }
    else {
      this.$membersAnswers.show();
    }
    this.$pauseText.hide();

    this.trialTextArray[this.i+1].show();
    this.$trialCounter.show();
    //this.$textContainerBottom.hide();
    this.$textContainer.hide();
    $('.button_container').show();

    document.onkeypress = this.keyListener.bind( this );
    this.showTrial();
  };

  Trial.prototype.saveResult = function() {
    this.$trialCounter.hide();
    this.trialTextArray[20].hide();
    this.$membersAnswers.hide();
    this.$textContainerTop.hide();
    $('.text-container-top2').hide();
    this.$closeButton.show();
    $('.button_container').hide();
    for ( var i = 0; i < 10; i++ ) {
        this.results[ 'self' ]+= this.answers[i];
    }
    this.results[ 'self' ]=this.results[ 'self' ]/10;
    for ( i = 10; i < 20; i++ ) {

        this.results[ 'other' ]+= this.answers[i];
    }
    this.results[ 'other' ]= this.results[ 'other' ]/10;
    window.results = this.results;

    setTimeout(function() {
      $("#close button").focus();
    }, 1000);

    /*setTimeout(() => {
      this.liveFeedback.text('');
    }, 3000);*/
  };

  Trial.prototype.showTrial = function() {
    if ( this.i < this.trialNum ) {
      var html = '';
      this.$membersAnswers.html( html );
      this.i++;
      this.$trialCounterCurrent.text( this.i );
      if (this.i > 1) {
        this.trialTextArray[this.i-1].hide();
      }
      this.trialTextArray[this.i].show();

      if (window.platform.iOS) {
        this.trialTextArray[this.i].removeAttr('aria-label');
        this.liveFeedback.html();
        this.liveFeedback.html('Trial ' + this.i + ' of ' + this.trialNum + this.trialTextArray[this.i].text());
      }
      else {
        this.trialTextArray[this.i].attr('aria-label', 'Trial ' + this.i + ' of ' + this.trialNum + '. ' + this.trialTextArray[this.i].text());
      }

      setTimeout(function () {
        this.trialTextArray[this.i].focus();
      }.bind(this), 0);
    }
    else {
      if ( !this.answered ) {
        this.saveResult();
        this.answered = true;
      }
    }
  };

  Trial.prototype.addAnswer = function( answer ) {
    this.answers.push( +answer );
  };

  Trial.prototype.keyListener = function( event ) {
    var keyCode = event.keyCode || event.which;
    var str;

    str = String.fromCharCode( keyCode );

    if ( '99887766554433221100'.indexOf( str ) > -1 ) {
      ( str === '9' || str === '9' ) && ( this.addAnswer( 9 ) );
      ( str === '8' || str === '8' ) && ( this.addAnswer( 8 ) );
      ( str === '7' || str === '7' ) && ( this.addAnswer( 7 ) );
      ( str === '6' || str === '6' ) && ( this.addAnswer( 6 ) );
      ( str === '5' || str === '5' ) && ( this.addAnswer( 5 ) );
      ( str === '4' || str === '4' ) && ( this.addAnswer( 4 ) );
      ( str === '3' || str === '3' ) && ( this.addAnswer( 3 ) );
      ( str === '2' || str === '2' ) && ( this.addAnswer( 2 ) );
      ( str === '1' || str === '1' ) && ( this.addAnswer( 1 ) );
      ( str === '0' || str === '0' ) && ( this.addAnswer( 0 ) );
      if ( this.i === 10 ) {
        this.pause();
      }
      else {
        this.showTrial();
      }
    }
  };

  var trial = new Trial();

  trial.init();

})();