// Extend Object constructor for custom events
(function() {

  var TOUCH = window.soel.libs['browser'].platform.Mobile;

  var CollectionOfListeners = {/* Collection of EventName: [ Listeners ] */};

  CollectionOfListeners.get = function( name ) {
    // If collection not existed - create, than return collection
    return ( ( !this[ name ] && ( this[ name ] = [] ) ) || this[ name ] );
  };

  var removeCustomEventListener = function( event, func ) {
    var coll = CollectionOfListeners.get( event );
    coll.splice( coll.indexOf( func ), 1 );
  };

  var addCustomEventListener = function( event, func ) {
    var coll = CollectionOfListeners.get( event );
    func && func.call && coll.push( func );
  };

  var dispatchCustomEvent = function( event, options ) {
    var coll = CollectionOfListeners.get( event );
    for ( var i = coll.length; i--; ) {
      coll[ i ]( options );
    }
  };

  var getCursorPosition = function( e ) {
    var x = e.pageX, y = e.pageY, html;
    if ( e.pageX == null && e.clientX != null ) {
      html = document.documentElement.body || document.body;
      x = e.clientX + ( html && html.scrollLeft || document.body && document.body.scrollLeft || 0 ) - ( html.clientLeft || 0 );
      y = e.clientY + ( html && html.scrollTop || document.body && document.body.scrollTop || 0 ) - ( html.clientTop || 0 );
    }
    if ( e.changedTouches && e.changedTouches.length ) {
      x = e.changedTouches[0].pageX;
      y = e.changedTouches[0].pageY;
    }
    return { x: x, y: y };
  };

  var getNormalizedCursorPosition = function( e, offset, scale ) {
    var position = getCursorPosition( e );
    position.y = (position.y - offset.y) / scale;
    position.x = (position.x - offset.x) / scale;

    return position;
  };

  var Event = function() {
    // Detect platform type
    this.user = {
      mousedown: ( TOUCH ) ? "touchstart" : "mousedown",
      mousemove: ( TOUCH ) ? "touchmove" : "mousemove",
      mouseup: ( TOUCH ) ? "touchend" : "mouseup",
      mouseover: ( TOUCH ) ? "xxx" : "mouseover",
      mouseout: ( TOUCH ) ? "xxx" : "mouseout",
      position: getCursorPosition,
      normalizedPosition: getNormalizedCursorPosition,
      touch: TOUCH
    };

    this.removeCustomEventListener = removeCustomEventListener;
    this.addCustomEventListener = addCustomEventListener;
    this.dispatchCustomEvent = dispatchCustomEvent;
  };

  window.soel = window.soel || {};
  window.soel.libs = window.soel.libs || {};

  window.soel.libs['event'] = window.soel.libs['event'] || new Event();

})();